/*
 * $Header: /home/cvs/jakarta-taglibs/application/src/org/apache/taglibs/application/AttributesTag.java,v 1.3 2002/11/10 15:16:29 glenn Exp $
 * $Revision: 1.3 $
 * $Date: 2002/11/10 15:16:29 $
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 */

package org.apache.taglibs.application;

import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;
import javax.servlet.jsp.*;
import javax.servlet.jsp.tagext.*;

/**
 * JSP Tag <b>attributes</b>, used to get ServletContext attribute
 * information using the standard JSP 1.2 &lt;jsp:getProperty&gt; tag.
 * <p>
 * The script variable of name <b>id</b> is availble only within the
 * body of the <b>attributes</b> tag.
 * <p>
 * Loops through all the attributes in the ServletContext.
 * <p>
 * JSP Tag Lib Descriptor
 * <p><pre>
 * &lt;name&gt;attributes&lt;/name&gt;
 * &lt;tagclass&gt;org.apache.taglibs.application.AttributesTag&lt;/tagclass&gt;
 * &lt;teiclass&gt;org.apache.taglibs.application.AttributesTEI&lt;/teiclass&gt;
 * &lt;bodycontent&gt;JSP&lt;/bodycontent&gt;
 * &lt;info&gt;Loop through all attributes or get a single attribute.&lt;/info&gt;
 *   &lt;attribute&gt;
 *     &lt;name&gt;id&lt;/name&gt;
 *     &lt;required&gt;true&lt;/required&gt;
 *     &lt;rtexprvalue&gt;false&lt;/rtexprvalue&gt;
 *   &lt;/attribute&gt;
 * </pre>
 *
 * @author Morgan Delagrange
 */

public class AttributesTag extends BodyTagSupport
{
    private String name = null;
    private ServletContext app = null;
    private Enumeration attributes = null;
    private String attribute = null;

    /**
     * Loops through all the attributes that came within the ServletContext.
     *
     * @return SKIP_BODY if no attributes are found, EVAL_BODY_TAG if an attribute exists
     */
    public final int doStartTag() throws JspException
    {
	// Get the ServletContext
	app = pageContext.getServletContext();
        attribute = null;

        if( name != null ) {
            if( app.getAttribute(name) != null) {
                attribute = name;
            }
        } else {
            attributes = app.getAttributeNames();
            if( attributes == null || !attributes.hasMoreElements() )
                return SKIP_BODY;
            attribute = (String)attributes.nextElement();
        }
	if( attribute == null )
	    return SKIP_BODY;

	pageContext.setAttribute(id,this,PageContext.PAGE_SCOPE);
	return EVAL_BODY_TAG;
    }

    /**
     * Method called at end of each attributes tag.
     *
     * @return EVAL_BODY_TAG if there is another attribute, or SKIP_BODY if there are no more attributes
     */
    public final int doAfterBody() throws JspException
    {
	// See if this is the last attribute
	if( name != null || !attributes.hasMoreElements() )
	    return SKIP_BODY;
	// There is another attribute, so loop again
	attribute = (String)attributes.nextElement();
	if( attribute == null )
	    return SKIP_BODY;
	return EVAL_BODY_TAG;
    }

    /**
     * Method called at end of Tag
     * @return EVAL_PAGE
     */
    public final int doEndTag() throws JspException
    {
        pageContext.removeAttribute(id,PageContext.PAGE_SCOPE);
	try
	{
	    if(bodyContent != null)
	    bodyContent.writeOut(bodyContent.getEnclosingWriter());
	} catch(java.io.IOException e)
	{
	    throw new JspException("IO Error: " + e.getMessage());
	}
	return EVAL_PAGE;
    }

    /**
     * Returns the name of the attribute.
     * <p>
     * &lt;jsp:getProperty name=<i>"id"</i> property="name"/&gt;
     *
     * @return String - attribute name
     */
    public final String getName()
    {
	return attribute;
    }

    /**
     * Set the name of the attribute to get
     *
     * @param String - attribute name
     */
    public final void setName(String name)
    {
        this.name = name;
    }

    /**
     * Returns the value of the attribute.
     * <p>
     * &lt;jsp:getProperty name=<i>"id"</i> property="value"/&gt;
     *
     * @return String - value of the attribute
     */
    public final String getValue()
    {
	Object value = app.getAttribute(attribute);
	if( value == null )
	    return "";
	return "" + value.toString();
    }

}
