/*
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:  
 *       "This product includes software developed by the 
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 */ 

package org.apache.taglibs.cache;

import javax.servlet.jsp.*;
import javax.servlet.jsp.tagext.*;
import org.apache.taglibs.standard.lang.support.ExpressionEvaluatorManager;

/**
 * <p>Invalidates a particular cache entry, or an entire cache.</p>
 *
 * @author Shawn Bayern
 */

public class InvalidateTag extends TagSupport {

    //*********************************************************************
    // Private state

    String nameExpr;					// tag attribute
    String keyExpr;					// tag attribute

    int scope;						// parsed tag attribute
    String name, key;					// parsed tag attribute

    //*********************************************************************
    // Tag logic

    public int doStartTag() throws JspException {
      evaluateExpressions();
      if (keyExpr == null || key == null) {
        // invalidate whole cache
        CacheUtil.invalidateCache(scope, name, pageContext);
      } else {
        // invalidate cache item
        CacheUtil.invalidateCachedItem(scope, name, key, pageContext);
      }
      return SKIP_BODY;
    }

    //*********************************************************************
    // Attribute accessors

    public void setScope(String scope) {
      if (scope.equalsIgnoreCase("page"))
	this.scope = PageContext.PAGE_SCOPE;
      else if (scope.equalsIgnoreCase("request"))
        this.scope = PageContext.REQUEST_SCOPE;
      else if (scope.equalsIgnoreCase("session"))
        this.scope = PageContext.SESSION_SCOPE;
      else if (scope.equalsIgnoreCase("application"))
        this.scope = PageContext.APPLICATION_SCOPE;
      else
        throw new IllegalArgumentException("invalid scope");
    }

    public void setName(String nameExpr) {
      this.nameExpr = nameExpr;
    }

    public void setKey(String keyExpr) {
      this.keyExpr = keyExpr;
    }


    //*********************************************************************
    // Constructor and initialization

    public InvalidateTag() {
      super();
      init();
    }

    private void init() {
      scope = PageContext.APPLICATION_SCOPE;
      name = nameExpr = null;
      key = keyExpr = null;
    }
    

    //*********************************************************************
    // Private utility methods

    private void evaluateExpressions() throws JspException {
      if (nameExpr != null)
        name = (String) ExpressionEvaluatorManager.evaluate(
          "name",
          nameExpr,
          String.class,
          this,
          pageContext);
      if (keyExpr != null)
        key = (String) ExpressionEvaluatorManager.evaluate(
          "key",
          keyExpr,
          String.class,
          this,
          pageContext);
    }

}
