/*
 * $Header: /home/cvs/jakarta-taglibs/datetime/src/org/apache/taglibs/datetime/DateFormatSymbolsTag.java,v 1.2 2003/01/27 14:12:43 glenn Exp $
 * $Revision: 1.2 $
 * $Date: 2003/01/27 14:12:43 $
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 */

package org.apache.taglibs.datetime;

import java.util.Locale;
import java.util.ResourceBundle;
import java.util.MissingResourceException;
import java.text.DateFormatSymbols;
import javax.servlet.jsp.tagext.TagSupport;
import javax.servlet.jsp.PageContext;
import javax.servlet.jsp.JspException;

/**
 * <p>
 * This class implements an empty tag that allows you to use a custom symbols
 * for date formating.</p>
 * <p>
 * Must be specified attribute baseName that point to property file.
 * </p>
 * <p>
 * Property file contains key\value pair in such rule<br>
 * # Month symbols change on custom<br>
 * month=true<br>
 * month0=January<br>
 * month1=February<br>
 * # Era symbols change on custom<br>
 * era=true<br>
 * era0=before<br>
 * era1=after<br>
 *
 * </p>
 *
 * @author <a href="mailto:wigor@bigmir.net">Kozlov Igor</a>
 */
public class DateFormatSymbolsTag extends TagSupport {

    private int scope = PageContext.PAGE_SCOPE;
    private String localeRef = null;
    private boolean locale_flag = false;
    private String baseName = null;

    private ResourceBundle bundle;
    private DateFormatSymbols dateFormatSymbols;

    public void setScope(String scope) {
        if (scope.toLowerCase().equals("application")) {
            this.scope = PageContext.APPLICATION_SCOPE;
        } else if (scope.toLowerCase().equals("session")) {
            this.scope = PageContext.SESSION_SCOPE;
        } else if (scope.toLowerCase().equals("request")) {
            this.scope = PageContext.REQUEST_SCOPE;
        } else if (scope.toLowerCase().equals("page")) {
            this.scope = PageContext.PAGE_SCOPE;
        }
    }

    public void setBaseName(String baseName) {
        this.baseName = baseName;
    }

    public void setLocale(boolean locale) {
        this.locale_flag = locale;
    }

    public void setLocaleRef(String localeRef) {
        this.localeRef = localeRef;
    }

    public int doStartTag() throws JspException {
        Locale locale = null;
        if (localeRef != null) {
            locale = (Locale) pageContext.findAttribute(localeRef);
        } else if (locale_flag) {
            locale = pageContext.getRequest().getLocale();
        } else {
            dateFormatSymbols = new DateFormatSymbols();
        }
        findBundle(locale);
        buildDateFormatSymbols(locale);
        if (this.getId() != null) {
            pageContext.setAttribute(this.getId(), dateFormatSymbols, scope);
        }
        return SKIP_BODY;
    }

    public void release() {
        super.release();
        scope = PageContext.PAGE_SCOPE;
        localeRef = null;
        locale_flag = false;
        baseName = null;
    }

    private void buildDateFormatSymbols(Locale locale) {
        if (locale != null) {
            dateFormatSymbols = new DateFormatSymbols(locale);
        } else {
            dateFormatSymbols = new DateFormatSymbols();
        }
        // Set ampm string
        try {
            String customAmPm = bundle.getString("ampm");
            if ("true".equals(customAmPm)) {
                String ampm[] = new String[2];
                for (int i = 0; i < 2; i++) {
                    ampm[i] = bundle.getString("ampm" + i);
                }
                dateFormatSymbols.setAmPmStrings(ampm);
            }
        } catch (MissingResourceException e) {
        }

        // Set era strings
        try {
            String customEras = bundle.getString("eras");
            if ("true".equals(customEras)) {
                String eras[] = new String[2];
                for (int i = 0; i < 2; i++) {
                    eras[i] = bundle.getString("eras" + i);
                }
                dateFormatSymbols.setEras(eras);
            }
        } catch (MissingResourceException e) {
        }

        // Set short month strings
        try {
            String customShortMonth = bundle.getString("shortmonth");
            if ("true".equals(customShortMonth)) {
                String shortmonth[] = new String[12];
                for (int i = 0; i < 12; i++) {
                    shortmonth[i] = bundle.getString("shortmonth" + i);
                }
                dateFormatSymbols.setShortMonths(shortmonth);
            }
        } catch (MissingResourceException e) {
        }

        // Set month strings
        try {
            String customMonth = bundle.getString("month");
            if ("true".equals(customMonth)) {
                String month[] = new String[12];
                for (int i = 0; i < 12; i++) {
                    month[i] = bundle.getString("month" + i);
                }
                dateFormatSymbols.setMonths(month);
            }
        } catch (MissingResourceException e) {
        }

        // Set weekdays strings
        try {
            String customWeekdays = bundle.getString("weekdays");
            if ("true".equals(customWeekdays)) {
                String weekdays[] = new String[7];
                for (int i = 0; i < 7; i++) {
                    weekdays[i] = bundle.getString("weekdays" + i);
                }
                dateFormatSymbols.setWeekdays(weekdays);
            }
        } catch (MissingResourceException e) {
        }

        // Set short weekdays strings
        try {
            String customShortWeekdays = bundle.getString("shortweekdays");
            if ("true".equals(customShortWeekdays)) {
                String shortweekdays[] = new String[7];
                for (int i = 0; i < 7; i++) {
                    shortweekdays[i] = bundle.getString("shortweekdays" + i);
                }
                dateFormatSymbols.setShortWeekdays(shortweekdays);
            }
        } catch (MissingResourceException e) {
        }
    }

    private void findBundle(Locale locale) {
        if (locale != null) {
            ClassLoader cl = Thread.currentThread().getContextClassLoader();
            bundle = ResourceBundle.getBundle(baseName, locale, cl);
        } else {
            bundle = ResourceBundle.getBundle(baseName);
        }
    }

}
