/*
 * $Header: /home/cvs/jakarta-taglibs/datetime/src/org/apache/taglibs/datetime/TimeZonesTag.java,v 1.5 2002/04/06 18:10:51 glenn Exp $
 * $Revision: 1.5 $
 * $Date: 2002/04/06 18:10:51 $
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 */

package org.apache.taglibs.datetime;

import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;
import javax.servlet.jsp.*;
import javax.servlet.jsp.tagext.*;

/**
 * JSP Tag <b>timeZones</b>, used to loop through all the TimeZone's
 * so that ID's and Display Names can be accessed by using the standard
 * JSP &lt;jsp:getProperty&gt; tag.
 * <p>
 * The script variable of name <b>id</b> is availble only within the
 * body of the <b>timeZones</b> tag.
 * <p>
 * Loops through all the timeZones.
 * <p>
 * If the optional attribute <b>locale</b> is <b>true</b>, the Displaynames
 * are formatted for the clients locale if known.
 * <p>                                                         
 * The optional attribute <b>localeRef</b> can be used to specify
 * the name of a page, session, application, or request scope attribute
 * of type java.util.Locale to use.
 * <p>
 * The optional attribute <b>style</b> can be set to <i>SHORT</i> or
 * <i>LONG</i>.
 * <p>
 * JSP Tag Lib Descriptor
 * <p><pre>
 * &lt;name&gt;timeZones&lt;/name&gt;
 * &lt;tagclass&gt;org.apache.taglibs.datetime.TimeZonesTag&lt;/tagclass&gt;
 * &lt;teiclass&gt;org.apache.taglibs.datetime.TimeZonesTEI&lt;/teiclass&gt;
 * &lt;bodycontent&gt;JSP&lt;/bodycontent&gt;
 * &lt;info&gt;Loop through all the timeZone display names.&lt;/info&gt;
 *   &lt;attribute&gt;
 *     &lt;name&gt;id&lt;/name&gt;
 *     &lt;required&gt;true&lt;/required&gt;
 *     &lt;rtexprvalue&gt;false&lt;/rtexprvalue&gt;
 *   &lt;/attribute&gt;
 *   &lt;attribute&gt;
 *     &lt;name&gt;locale&lt;/name&gt;
 *     &lt;required&gt;false&lt;/required&gt;
 *     &lt;rtexprvalue&gt;false&lt;/rtexprvalue&gt;
 *   &lt;/attribute&gt;
 *   &lt;attribute&gt;                             
 *     &lt;name&gt;localeRef&lt;/name&gt;
 *     &lt;required&gt;false&lt;/required&gt;
 *     &lt;rtexprvalue&gt;false&lt;/rtexprvalue&gt;
 *   &lt;/attribute&gt;
 *   &lt;attribute&gt;
 *     &lt;name&gt;style&lt;/name&gt;
 *     &lt;required&gt;false&lt;/required&gt;
 *     &lt;rtexprvalue&gt;false&lt;/rtexprvalue&gt;
 *   &lt;/attribute&gt;
 * </pre>
 *
 * @author Glenn Nielsen
 */

public class TimeZonesTag extends BodyTagSupport
{
    // timeZones tag attributes
    private boolean locale_flag = false;
    private String localeRef = null;
    private String style_string = "SHORT";

    // timeZones tag invocation variables
    private int style = TimeZone.SHORT;
    private String [] timeZones = null;
    private TimeZone timeZone = null;
    private int zone_num = 0;

    /**
     * Initializes tag so it can loop through the time zones.
     *
     * @return EVAL_BODY_TAG, or SKIP_BODY if no time zones are found
     */
    public final int doStartTag() throws JspException
    {
        // Initialize variables                      
        zone_num = 0;

        if( style_string == null || style_string.equals("SHORT") )  
            style = TimeZone.SHORT;
        else if( style_string.equals("LONG") )
            style = TimeZone.LONG;
        else
           throw new JspTagException(
               "Datetime tag timeZones style attribute must be set to" +
               " either SHORT or LONG");
	timeZones = TimeZone.getAvailableIDs();

	timeZone = TimeZone.getTimeZone(timeZones[zone_num]);
	if( timeZone == null )
	    return SKIP_BODY;

	pageContext.setAttribute(id,this,PageContext.PAGE_SCOPE);
	return EVAL_BODY_TAG;
    }

    /**
     * Method called at end of each timeZones tag.
     *
     * @return EVAL_BODY_TAG if there is another timeZone, or SKIP_BODY if there are no more timeZones
     */
    public final int doAfterBody() throws JspException
    {
	// See if we are done looping through timeZones
	zone_num++;
	if( zone_num >= timeZones.length )
	    return SKIP_BODY;

	// There is another timeZone, so loop again
	timeZone = TimeZone.getTimeZone(timeZones[zone_num]);
	if( timeZone == null )
	    return SKIP_BODY;

	return EVAL_BODY_TAG;
    }

    /**
     * Method called at end of Tag
     *
     * @return EVAL_PAGE
     */
    public final int doEndTag() throws JspException
    {
        pageContext.removeAttribute(id,PageContext.PAGE_SCOPE);
	try
	{
	    if(bodyContent != null)
	    bodyContent.writeOut(bodyContent.getEnclosingWriter());
	} catch(java.io.IOException e)
	{
	    throw new JspException("IO Error: " + e.getMessage());
	}
	return EVAL_PAGE;
    }

    /**
     * Locale flag, if set to true, format timeZone Displayname
     * for client's preferred locale if known.
     *
     * @param boolean either <b>true</b> or <b>false</b>
     */
    public final void setLocale(boolean flag)
    {
        locale_flag = flag;
    }

    /**
     * Provides a key to search the page context for in order to get the
     * java.util.Locale to use.
     *
     * @param String name of locale attribute to use
     */
    public void setLocaleRef(String value)
    {
        localeRef = value;
    }

    /**
     * Set they style of Displaynames to either <b>SHORT</b> or <b>LONG</b>.
     *
     * @param String style, either <b>SHORT</b> or <b>LONG</b>
     */
    public final void setStyle(String str)
    {
        style_string = str;
    }

    /**
     * Returns the display name of the timeZone.
     * <p>
     * &lt;jsp:getProperty name=<i>"id"</i> property="displayName"/&gt;
     *
     * @return String - display name
     */
    public final String getDisplayName() throws JspException
    {
	String dn = null;
	Date now = new Date();
	boolean daylight = false;

	if( timeZone.useDaylightTime() )
	    daylight = timeZone.inDaylightTime(now);

        if( localeRef != null ) {                          
            Locale locale = (Locale)pageContext.findAttribute(localeRef);
            if( locale == null ) {
                throw new JspException(
                    "datetime amPms tag could not find locale for localeRef \"" +
                    localeRef + "\".");
            }
            dn = timeZone.getDisplayName(daylight,style,locale);
	} else if( locale_flag) {
	    dn = timeZone.getDisplayName(daylight,style,
                     (Locale)pageContext.getRequest().getLocale());
	} else {
	    dn = timeZone.getDisplayName(daylight, style);
	}

	if( dn == null )
	    dn = "";

	return dn;
    }

    /**
     * Returns the value of the time zone ID.
     * <p>
     * &lt;jsp:getProperty name=<i>"id"</i> property="zoneId"/&gt;
     *
     * @return String - time zone ID
     */
    public final String getZoneId()
    {
	return timeZone.getID();
    }

}
