/*
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 */
package org.apache.taglibs.dbtags.resultset;

import java.sql.Clob;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.Types;
import java.sql.SQLException;

import java.io.IOException;
import java.io.BufferedReader;
import java.io.Reader;

import javax.servlet.jsp.JspTagException;

/**
 * <p>Get the value of a database column as a String.</p>
 * 
 * <p>JSP Tag Lib Descriptor
 * <pre>
 * &lt;name>getString&lt;/name>
 * &lt;tagclass>org.apache.taglibs.dbtags.resultset.GetStringTag&lt;/tagclass>
 * &lt;bodycontent>empty&lt;/bodycontent>
 * &lt;info>Gets the value, as a String, of a coulmn in the enclosing
 * resultset.  Either set the column number via the "position" attribute,
 * or set the column name with the "colName" attribute.
 * You can optionally set the value, as a String, to a serlvet attribute 
 * instead of the tag body with the "to" attribute.  The scope of the servlet
 * attribute is specified by the "scope" XML attribute (default = page).  Dates,
 * times, timestamps and numbers are output according to the JVM's defaults.&lt;/info>
 *   &lt;attribute>
 *     &lt;name>position&lt;/name>
 *     &lt;required>false&lt;/required>
 *     &lt;rtexprvalue>false&lt;/rtexprvalue>
 *   &lt;/attribute>
 *     &lt;attribute>
 *     &lt;name>colName&lt;/name>
 *     &lt;required>false&lt;/required>
 *     &lt;rtexprvalue>false&lt;/rtexprvalue>
 *   &lt;/attribute>
 *   &lt;attribute>
 *     &lt;name>to&lt;/name>
 *     &lt;required>false&lt;/required>
 *     &lt;rtexprvalue>false&lt;/rtexprvalue>
 *   &lt;/attribute>
 *     &lt;attribute>
 *     &lt;name>scope&lt;/name>
 *     &lt;required>false&lt;/required>
 *     &lt;rtexprvalue>false&lt;/rtexprvalue>
 *   &lt;/attribute>
 * </pre>
 * 
 * @author Morgan Delagrange
 */
public class GetColumnTag extends BaseGetterTag {
    
  public int doStartTag() throws JspTagException {
    try {
      int position = getPosition();
      
      ResultSet rset = getResultSet();
      
      // some complex datatypes, such as clobs,
      // require special handling
      ResultSetMetaData meta = getMetaData();
      String string = null;
      switch (meta.getColumnType(position)) {
      case (Types.CLOB):
        try {
          string = readClob(rset.getClob(position));
        } catch (IOException e) {
          throw new JspTagException(e.toString());
        } catch (SQLException e) {
          throw new JspTagException(e.toString());
        }
        break; 
      default:
        string = rset.getString(position);
      }
      
      // null results are often OK, in outer joins for example
      if (string == null) {
        return EVAL_BODY_INCLUDE;
      }

      if (_attributeName != null) {
        setAttribute(_attributeName, string, _scope);
      } else {
        pageContext.getOut().write(string);
      }
    } catch (SQLException e) {
      throw new JspTagException(e.toString());
    } catch (IOException e) {
      throw new JspTagException(e.toString());
    }
    
    return EVAL_BODY_INCLUDE;
  }

  private String readClob(Clob clob) throws IOException, SQLException {
    StringBuffer buffer = new StringBuffer();
    Reader reader = clob.getCharacterStream();
    BufferedReader buffReader = new BufferedReader(reader);
    String line = buffReader.readLine();
    while (line != null) {
      buffer.append(line);
      line = buffReader.readLine();
    }
    buffReader.close();
    return buffer.toString();
  }
}
