
/*
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 */
package org.apache.taglibs.dbtags.resultset;

import javax.servlet.jsp.JspTagException;
import javax.servlet.jsp.tagext.TagSupport;

/**
 * <p>Executes its body if the last ResultSet tag received 0 rows
 * from the database.  You must be after a ResultSet tag and inside
 * a StatementTag or PreparedStatementTag, or an error will be generated.</p>
 *
 * <p>The subclass WasNotEmpty sets the "value" property to false,
 * and therefore executes its tag body is the ResultSet contained
 * <i>greater than</i> 0 rows.  
 * 
 * <p>JSP Tag Lib Descriptor
 * <pre>
 * &lt;name>wasEmpty&lt;/name>
 * &lt;tagclass>org.apache.taglibs.dbtags.statement.WasEmptyTag&lt;/tagclass>
 * &lt;bodycontent>JSP&lt;/bodycontent>
 * &lt;info>
 * Executes its body if the last ResultSet tag received 0 rows
 * from the database.  You must be after a ResultSet tag and inside
 * a StatementTag or PreparedStatementTag, or an error will be generated.
 * &lt;/info>
 * </pre>
 * 
 * @author Morgan Delagrange
 * @see WasNotEmptyTag
 */
public class WasEmptyTag extends TagSupport {

  private boolean _value = true;

  /**
   * Sets the necessary boolean value in order to
   * execute the tag body.
   * 
   * @param value
   */
  public void setValue(boolean value) {
    _value = value;
  }

  public int doStartTag() throws JspTagException {

    Integer integer = 
      (Integer) pageContext.getAttribute("org.apache.taglibs.dbtags.resultset.rowcount");
    
    if (integer == null) {
      throw new JspTagException("WasEmpty and WasNotEmpty tags must follow a ResultSet tag.");
    }

    int rowCount = integer.intValue();

    boolean wasEmpty = true;
    if (rowCount > 0) {
      wasEmpty = false;
    }

    // evaluate the body only if wasEmpty matches the desired
    // value (true or false)
    if (wasEmpty == _value) {
      return EVAL_BODY_INCLUDE;
    } else {
      return SKIP_BODY;
    }
  }

  public void release() {
    _value = true;
  }
}

