/*
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 */
package org.apache.taglibs.dbtags.statement;

import java.sql.ResultSet;
import java.sql.SQLException;

import javax.servlet.jsp.JspTagException;

/**
 * <p>Interface for StatementImplTag and PreparedStatementImplTag.
 * Used to mask the differences between statements and preparedstatements
 * in this taglib.</p>
 * 
 * @author Morgan Delagrange
 * @see StatementImplTag
 * @see org.apache.taglibs.dbtags.preparedstatement.PreparedStatementImplTag
 * @see QueryTag
 * @see ExecuteTag
 * @see org.apache.taglibs.dbtags.resultset.ResultSetTag
 */
public interface StatementTag {
  
  /**
   * SQL query to be executed in the statement
   * 
   * @param query  SQL query
   * @exception SQLException
   *                   throws an exception when a PreparedStatement cannot be created
   */
  public void setQuery(String query) throws SQLException, JspTagException;

  /**
   * The id of a page context attribute containing a java.sql.Connection
   * 
   * @param connId id of the Connection attribute
   * @see org.apache.taglibs.dbtags.connection.ConnectionTag
   */
  public void setConn(String connId);

  /**
   * Execute a SQL insert, update or delete.
   * 
   * @exception SQLException
   */
  public void executeUpdate() throws SQLException;

  /**
   * Execute a SQL select
   * 
   * @return Resultset based on the {@link #setQuery query}
   * @exception SQLException
   */
  public ResultSet executeQuery() throws SQLException;

  
}
