/*
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 */

package org.apache.taglibs.gnat;

import org.apache.taglibs.gnat.util.*;
import javax.servlet.jsp.*;
import javax.servlet.jsp.tagext.*;
import java.io.*;
import java.util.*;

public class echoTag extends TagSupport
{
    private String message=null;
    private File f = null;    // real file
    private String file = null; // "file" attribute value
    private boolean append = false;
    private ResourceBundle gnatRB = ListResourceBundle.getBundle("org.apache.taglibs.gnat.util.GnatTagStrings");
    private ResourceBundle gnatERB = ListResourceBundle.getBundle("org.apache.taglibs.gnat.util.GnatExceptionStrings");
        
    public void setMessage(String msg) 
    {
        this.message = msg;
    }

    public void setFile(String file) 
    {
        this.file = file;
    }

    /**
    * Shall we append to an existing file?
    */
    public void setAppend(boolean append) 
    {
        this.append = append;
    }

    
    public int doStartTag() throws JspException
    {
        if (message == null)
        {
            return EVAL_BODY_INCLUDE;
        }
        else
        return SKIP_BODY;
    }
   
    /**
    * "Echoes" message to the JSP's output stream.
    */ 
    public int doEndTag() throws JspException
    {
        if (file != null)
        {
            f = new File(file);
            
            if (f.isDirectory())
            {
                throw new JspTagException( gnatRB.getString("echo.tag") + ": " +
                                           gnatERB.getString("echo.dir.fail") );
            }
            
            FileWriter out = null;
            try 
            {
                out = new FileWriter( f.getAbsolutePath(), append );
                out.write( message, 0, message.length() );
            } 
            catch (IOException ioe) 
            {
                throw new JspTagException( gnatRB.getString("echo.tag") + ": " +
                                           ioe.getMessage() );
            } 
            finally 
            {
                if (out != null) 
                {
                    try 
                    {
                        out.close();
                    } 
                    catch (IOException ioex) 
                    {
                        throw new JspTagException( gnatRB.getString("echo.tag") + ": " + 
                                                   ioex.getMessage() );
                    }
                }
            }
        }
        else if (message != null)
        {
            try
            {
                pageContext.getOut().write(message); 
            }
            catch(IOException ioe)
            {
                throw new JspTagException( gnatRB.getString("echo.tag") + ": " +
                                           ioe.getMessage() ); 
            }
        }
        return EVAL_PAGE;
    }

    /* Put tag attribute accessors down here, out of the way, since they're more for 
       JavaBean completeness than programmer use. 
     */
    public String getMessage() { return message; }

    public String getFile() { return file; }

    public boolean getAppend() { return append; }
}
