/*
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 */

package org.apache.taglibs.gnat;

import org.apache.taglibs.gnat.util.FileUtil;
import javax.servlet.ServletContext;
import javax.servlet.jsp.*;
import javax.servlet.jsp.tagext.*;
import java.io.*;
import java.util.zip.*;
import java.util.*;

public class gunzipTag extends TagSupport
{
    //Need to decide what to do with basedir.  Need to use it for full Ant support,
    //but for JSP taglib, it should be optional.  Perhaps a scripting variable.
    //For now, it's always null until we figure out a good way to use it.
    private String bdir = "";
    private File baseDir;
    private String dest; // the dest attribute
    private File destF;  // the dest file
    private String src;  // the src attribute
    private File source; // the src file
    private ServletContext ctx; // for logging
    private FileUtil fut; // contains some static util methods ripped from Avalon utils

    
    private void setDestF(String dest){ 

        destF = fut.resolveFile(null, dest);
    }

    private void setSource(String src) {

        source = fut.resolveFile(null, src);
    }
    
    public void setDest(String dest) {

        this.dest = dest;
    }

    public void setSrc(String src) {

        this.src = src;
    
    }
    
    public int doStartTag() throws JspException 
    {
        ctx = pageContext.getServletContext();
        
        /* Until we decide how to deal with projects in the Ant tags, bdir is 
         * not set.  So this if statement is always false, for the time being.
         */
        if (!bdir.equals(""))
        {
            baseDir = new File(src);
            //Should fix so that we call different FileUtil methods, depending 
            //whether a basefile name is passed.  Could be an optional attribute.
            setDestF(dest);
            return SKIP_BODY;
        }
        else
        {
            baseDir = new File(bdir);
            setSource(src);
            //Should fix so that we call different FileUtil methods, depending 
            //whether a basefile name is passed.  Could be an optional attribute.
            setDestF(dest);
        
            return SKIP_BODY;
        }
    }
    
    public int doEndTag() throws JspException 
    {
        if (source == null) 
        {
            throw new JspTagException("gunzip: No source file for gunzip specified.");
        }

        if (!source.exists()) 
        {
            throw new JspTagException("gunzip: Source file "+src+" doesn't exist.");
        }

        if (source.isDirectory()) 
        {
            throw new JspException("gunzip: Cannot expand a directory. "+src+" is a directory.");
        }

        if (destF == null) 
        {
            destF = new File(source.getParent());
        }

        if (destF.isDirectory()) 
        {
            String sourceName = source.getName();
            int len = sourceName.length();
            if (len > 3
                && ".gz".equalsIgnoreCase(sourceName.substring(len-3))) 
            {
                destF = new File(destF, sourceName.substring(0, len-3));
            } 
            else 
            {
                destF = new File(destF, sourceName);
            }
        }

        if (source.lastModified() > destF.lastModified()) 
        {
            ctx.log("gunzip: Expanding "+ source.getAbsolutePath() + " to "
                        + destF.getAbsolutePath());

            FileOutputStream out = null;
            GZIPInputStream zIn = null;
            try 
            {
                out = new FileOutputStream(destF);
                zIn = new GZIPInputStream(new FileInputStream(source));
                byte[] buffer = new byte[8 * 1024];
                int count = 0;
                do 
                {
                    out.write(buffer, 0, count);
                    count = zIn.read(buffer, 0, buffer.length);
                } 
                while (count != -1);
            } 
            catch (IOException ioe) 
            {
                String msg = "gunzip: Problem expanding gzip " + ioe.getMessage();
                throw new JspTagException(msg);
            } 
            finally 
            {
                if (out != null) 
                {
                    try 
                    {
                        out.close();
                    } 
                    catch (IOException ioex) 
                    {
                        ioex.printStackTrace();
                    }
                }
                if (zIn != null) 
                {
                    try 
                    {
                        zIn.close();
                    } 
                    catch (IOException ioex) 
                    {
                        ioex.printStackTrace();
                    }
                }
            }
        }
        return EVAL_PAGE;
    }

    public String getDest() {
        return dest;
    }

    public String getSrc() {
        return src;
    }
}
