/*
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 */

package org.apache.taglibs.gnat;

import org.apache.taglibs.gnat.util.FileUtil;
import javax.servlet.ServletContext;
import javax.servlet.jsp.*;
import javax.servlet.jsp.tagext.*;
import java.io.*;
import java.util.zip.*;
import java.util.*;

public class gzipTag extends TagSupport
{
    //Need to decide what to do with basedir.  Need to use it for full Ant support,
    //but for JSP taglib, it should be optional.  Perhaps a scripting variable.
    //For now, it's always null until we figure out a good way to use it.
    private String bdir = "";
    private File baseDir;
    private String zipfile;
    private File zf;
    private String src;
    private File source;
    private ServletContext ctx; 
    private FileUtil fut;

    
    private void setZF(String zipfile){ 

        zf = fut.resolveFile(null, zipfile);
    }

    private void setSource(String src) {

        source = fut.resolveFile(null, src);
    }
    
    public void setZipfile(String zipfile) {

        this.zipfile = zipfile;
    }

    public void setSrc(String src) {

        this.src = src;
    
    }
    
    public int doStartTag() throws JspException {
        ctx = pageContext.getServletContext();
        if (!bdir.equals(""))
        {
            baseDir = new File(src);
            //See notes in gunzipTag.java
            setZF(zipfile);
            return SKIP_BODY;
        }
        else
        {
            baseDir = new File(bdir);
            setSource(src);
            //See notes in gunzipTag.java
            setZF(zipfile);
            return SKIP_BODY;
        }
    }
    
    public int doEndTag() throws JspException {
        if (zf == null) 
        {
            throw new JspTagException("gzip: zipfile attribute is required");
        }

        if (source == null) 
        {
            throw new JspTagException("gzip: src attribute is required");
        }
        ctx.log("gzip: Building gzip: " + zf.getAbsolutePath());

        GZIPOutputStream zOut = null;
        try 
        {
            zOut = new GZIPOutputStream(new FileOutputStream(zf));

            if (source.isDirectory()) 
            {
                ctx.log ("gzip: Cannot Gzip a directory!");
            } 
            else 
            {
                zipFile(source, zOut);
            }
        } 
        catch (IOException ioe) 
        {
            String msg = "gzip: Problem gzipping file: " + ioe.getMessage();
            throw new JspTagException(msg);
        } 
        finally 
        {
            if (zOut != null) 
            {
                try 
                {
                    zOut.close();
                }
                catch (IOException e) 
                {
                    e.printStackTrace();
                }
            }
        }
        return EVAL_PAGE;
    }

    private void zipFile(InputStream in, GZIPOutputStream zOut)
        throws IOException
    {
        byte[] buffer = new byte[8 * 1024];
        int count = 0;
        do 
        {
            zOut.write(buffer, 0, count);
            count = in.read(buffer, 0, buffer.length);
        } 
        while (count != -1);
    }

    private void zipFile(File file, GZIPOutputStream zOut)
        throws IOException
    {
        FileInputStream fIn = new FileInputStream(file);
        try 
        {
            zipFile(fIn, zOut);
        } 
        finally 
        {
            fIn.close();
        }
    }

    public String getZipfile() {
        return zipfile;
    }

    public String getSrc() {
        return src;
    }
}
