/*
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 */

package org.apache.taglibs.gnat;

import org.apache.taglibs.gnat.util.*;
import javax.servlet.jsp.*;
import javax.servlet.jsp.tagext.*;
import java.io.*;
import java.util.*;

public class lsTag extends BodyTagSupport
{
    private String dir = "";
    private File _f;
    private ResourceBundle gnatRB = ListResourceBundle.getBundle("org.apache.taglibs.gnat.util.GnatTagStrings");
    private ResourceBundle gnatERB = ListResourceBundle.getBundle("org.apache.taglibs.gnat.util.GnatExceptionStrings");
    private String item = null;
    private String[] _files = null;
    private int _i = 0;
        
    public void setDir(String dir) {
        this.dir = dir;
    }
    
    public void setItem(String item) {
        this.item = item;
    }
    
    /**
     * Gets the file listing and kicks off the loop through all the file names 
     * in a directory.
     *
     * @return SKIP_BODY if no items are found, EVAL_BODY_TAG if items exist.
     */
    public final int doStartTag() throws JspException
    {
        if (!dir.equals(""))
        {   // Get directory using absolute file name (add capability for relative files?)
            _f = FileUtil.resolveFile( null, dir );
             
            try
            {
                _files = _f.list();
    	        if( _files == null || !( _files.length > 0 ) )
                {
	                return SKIP_BODY;
                }
                else
                {
    	            item = getNext( _files, _i );
                    _i++;
                }
                
                pageContext.setAttribute( id, this ); 
            }
            catch(Exception e)
            {
                throw new JspTagException(gnatRB.getString("ls.tag") + ": " + e.getMessage());
            }
	        return EVAL_BODY_TAG;
        }
        else
        {
            throw new JspTagException(gnatRB.getString("ls.tag") + ": " + gnatERB.getString("empty.dir.attribute"));
        }
    }

    /**
     * Takes control of the loop from doStartTag(),  It accesses the
     * variables set up there and sets the logic for the rest of the loop.
     *
     * @return EVAL_BODY_TAG if there is another list item, or SKIP_BODY if there are no more.
     */
    public final int doAfterBody() throws JspException
    {
	    if( _i < _files.length ) {
    	    item = getNext( _files, _i );
            _i++;
            return EVAL_BODY_TAG;
        }
        else { 
            return SKIP_BODY; 
        }
    }

    /**
     * Method called at end of Tag to write out the response.
     * @return EVAL_PAGE
     */
    public final int doEndTag() throws JspException
    {
	    try {
	        if( bodyContent != null )
            bodyContent.writeOut( bodyContent.getEnclosingWriter() );
        } 
        catch(IOException ioe)
        {
            throw new JspException( gnatRB.getString("ls.tag") + ": " + ioe.getMessage() );
        }
        return EVAL_PAGE;
    }
   
    /**
    * Returns a String from a given array and index. Called from doStartTag and
    * and doAfterBody.
    */ 
    private String getNext(String array[], int index)
    {
        String s = array[index];
        return s;
    }

    /**
     * Remove the id variable after ls tag is finished.
     */
    public final void release()
    {
	    if( id != null )
	    pageContext.removeAttribute( id, PageContext.PAGE_SCOPE );
        _i = 0;
    }

    /* Put tag attribute accessors down here, out of the way, since they're more for 
       JavaBean completeness than programmer use. 
     */
    public String getDir() { return dir; }
    public String getItem() { return item; }
}
