/*
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 */

package org.apache.taglibs.gnat;

import org.apache.taglibs.gnat.util.*;
import javax.servlet.ServletContext;
import javax.servlet.jsp.*;
import javax.servlet.jsp.tagext.*;
import java.io.*;
import java.util.*;

public class mkdirTag extends TagSupport
{
    //Note: in Ant's Mkdir task, dir is a File object passed in from a client class
    private File _f;
    private String dir = "";
    private boolean clobber= false;
    private ResourceBundle gnatRB = ListResourceBundle.getBundle("org.apache.taglibs.gnat.util.GnatTagStrings");
    private ResourceBundle gnatERB = ListResourceBundle.getBundle("org.apache.taglibs.gnat.util.GnatExceptionStrings");
    
    public void setDir(String dir)
    {
        this.dir = dir;
    }
    
    public void setClobber(boolean clobber)
    {
        this.clobber = clobber;
    }
    
    
    public int doStartTag() throws JspException
    {
        _f = new File(dir);
        return SKIP_BODY;
    }
    
    public int doEndTag() throws JspException
    {
        
        if (_f == null) 
        {
            throw new JspTagException( gnatRB.getString("mkdir.tag") + ": " +
                                       gnatERB.getString("empty.dir.attribute") );
        }
        
        ServletContext ctx = pageContext.getServletContext();
        if (clobber) //default is clobber == false
        {
            if (_f.exists()) 
            {   // log that we are overwriting the directory . . .
                ctx.log(gnatRB.getString("mkdir.tag") + ": " +"Overwriting dir " + _f.toString());
                try {    // . . . and delete it before recreating it..
                    FileUtil.forceDelete(_f);
                }
                catch(IOException ioe) {
                    throw new JspTagException( gnatRB.getString("mkdir.tag") + ": " +
                                               ioe.getMessage() );
                }
            }

            boolean result = _f.mkdirs();
            if (result == false) 
            {
                String msg = gnatRB.getString("mkdir.tag") + ": " + 
                             gnatERB.getString("unknown.mkdir.fail") + 
                             _f.getAbsolutePath(); 
                
                throw new JspTagException(msg);
            }
            ctx.log( gnatRB.getString("mkdir.tag") + ": " +
                     gnatRB.getString("mkdir.success") + 
                     _f.getAbsolutePath() );
        }
        else if(!clobber) //default is clobber == false
        {
            if (_f.exists()) // throw an exception if the file exists
            {
                throw new JspTagException( gnatRB.getString("mkdir.tag") + ": " +
                                           gnatERB.getString("duplicate.mkdir.fail") + 
                                           _f.getAbsolutePath() );
            }
            else
            {
            
                boolean result = _f.mkdirs();
                if (result == false) 
                {
                    String msg = gnatRB.getString("mkdir.tag") + ": " + 
                                 gnatERB.getString("unknown.mkdir.fail") + 
                                 _f.getAbsolutePath(); 
                
                    throw new JspTagException(msg);
                }
                ctx.log( gnatRB.getString("mkdir.tag") + ": " +
                         gnatRB.getString("mkdir.success") + 
                         _f.getAbsolutePath() );
            }
        }

        return EVAL_PAGE;
    }
    
    public String getDir() { return dir; }
    public boolean getClobber() { return clobber; }
}
