/*
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 */

package org.apache.taglibs.gnat;

import javax.servlet.ServletContext;
import javax.servlet.jsp.*;
import javax.servlet.jsp.tagext.*;
import java.lang.reflect.Method;
import java.io.*;
import java.text.*;
import java.util.*;

/**
 * Touch a file - corresponds to the Unix touch command.
 *
 * <p>If the file to touch doesn't exist, an empty one is
 * created. Setting the modification time of files is not supported in
 * JDK 1.1.
 *
 * @author <a href="mailto:stefan.bodewig@megabit.net">Stefan Bodewig</a> 
 * @author <a href="mailto:ssstirling@mediaone.net">Scott Stirling</a> 
 */

public class touchTag extends TagSupport
{
    private File _file;
    private long millis=0;
    private String file="";
    private String datetime = null;
    private ServletContext ctx; // for logging
    private ResourceBundle gnatRB = ListResourceBundle.getBundle("org.apache.taglibs.gnat.util.GnatTagStrings");
    private ResourceBundle gnatERB = ListResourceBundle.getBundle("org.apache.taglibs.gnat.util.GnatExceptionStrings");
        
    private static Method setLastModified = null;
    private static Object lockReflection = new Object();
    
    /**
     * The name of the file to touch.
     */   
    public void setFile(String file) { this.file = file; }

    /**
     * Milliseconds since 01/01/1970 00:00 am.
     */
    public void setMillis(long millis) { this.millis = millis; }

    /**
     * Date in the format MM/DD/YYYY HH:MM AM_PM.
     */
    public void setDatetime(String datetime) { this.datetime = datetime; }
    
    
    public int doStartTag() throws JspException
    {
        ctx = pageContext.getServletContext();
        _file = new File(file);
        
        return SKIP_BODY;
    }
    
    public int doEndTag() throws JspException
    {
        if (datetime != null) 
        {
            DateFormat df = DateFormat.getDateTimeInstance( DateFormat.SHORT,
                                                            DateFormat.SHORT,
                                                            Locale.getDefault() );
            try 
            {
                setMillis( df.parse(datetime).getTime() );
            } 
            catch (ParseException pe) 
            {
                throw new JspTagException( gnatRB.getString("touch.tag") + pe.getMessage() );
            }
            touch();
            return EVAL_PAGE;
        }
        else
        {
            touch();
            return EVAL_PAGE;
        }
    }

    /**
     * Does the actual work.
     */
    private void touch() throws JspTagException
    {
        if ( !_file.exists() ) {
            try 
            {
                FileOutputStream fos = new FileOutputStream(_file);
                fos.write(new byte[0]);
                fos.close();
            } 
            catch (IOException ioe) 
            {
                throw new JspTagException( gnatRB.getString("touch.tag") + ": " +
                                           gnatERB.getString("touch.no.create") + 
                                           ioe.getMessage() );
            }
            ctx.log( gnatRB.getString("touch.success") + file );
        }

        if (setLastModified == null) 
        {
            synchronized (lockReflection) 
            {
                if (setLastModified == null) 
                {
                    try
                    {
                        setLastModified =
                            java.io.File.class.getMethod( "setLastModified",
                                                          new Class[] { Long.TYPE } );
                    }
                    catch(NoSuchMethodException nsme)
                    {
                        throw new JspTagException( gnatRB.getString("touch.tag") +
                                                   gnatERB.getString("touch.nsme") +
                                                   nsme.getMessage() );
                    }
                }
            }
        }

        Long[] times = new Long[1];
        if (millis < 0) 
        {
            times[0] = new Long( System.currentTimeMillis() );
        } 
        else 
        {
            times[0] = new Long(millis);
        }
        try 
        {
            ctx.log( gnatRB.getString("touch.log.mod.time") + file );
            setLastModified.invoke( _file, times );
        } 
        catch (Throwable other) 
        {
            throw new JspTagException( gnatRB.getString("touch.tag") +
                                       gnatERB.getString("touch.throwable") +
                                       file );
        }
    }
   
    public String getFile() { return file; }
    
    public String getDatetime() { return this.datetime; }
    
    public long getMillis() { return this.millis; }
}
