/*
 * $Header: /home/cvs/jakarta-taglibs/i18n/src/org/apache/taglibs/i18n/ConditionalTagSupport.java,v 1.5 2002/12/19 04:33:05 glenn Exp $
 * $Revision: 1.5 $
 * $Date: 2002/12/19 04:33:05 $
 *
 * ====================================================================
 * 
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:  
 *       "This product includes software developed by the 
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 */ 

package org.apache.taglibs.i18n;

import java.io.IOException;

import java.util.ResourceBundle;

import javax.servlet.jsp.JspException;
import javax.servlet.jsp.JspTagException;
import javax.servlet.jsp.PageContext;
import javax.servlet.jsp.tagext.BodyTag;
import javax.servlet.jsp.tagext.BodyTagSupport;


/**
 *  This class provides the base implementation for the ifdef and ifndef tags.
 *  Subclasses must provide an implementation of the shouldEvaluate() method.
 *
 *  @author <a href="mailto:tdawson@wamnet.com">Tim Dawson</a>
 *
 */
public abstract class ConditionalTagSupport extends BodyTagSupport
{

    protected static String _tagname = "i18n:";

    // instance variables used during processing
    private String               _key = null;
    private ResourceBundle       _bundle = null;
    private String               _bundleRef = null;

    /**
     *  sets the key to use when looking up the value in the bundle
     */
    public void setKey(String key)
    {
        _key = key;
    }

    /**
     *  sets the bundle based on a variable defined in the page<br>
     *  if neither bundle or bundleRef are specified, uses the first bundle
     *  defined on the page by the i18n:bundle tag
     */
    public void setBundleRef(String varName)
    {
        _bundleRef = varName;
    }

    /**
     *  sets the bundle to an actual ResourceBundle object<br>
     *  if neither bundle or bundleRef are specified, uses the bundle most recently
     *  defined on the page by the i18n:bundle tag
     */
    public void setBundle(ResourceBundle aBundle)
    {
        _bundle = aBundle;
    }

    /**
     *  @return the bundle to use
     */
    public ResourceBundle getBundle()
    {
        if ( _bundleRef != null ) {
            _bundle = (ResourceBundle)pageContext.getAttribute(_bundleRef);
        }
        if ( _bundle == null ) {
            BundleTag bundleTag = (BundleTag)this.findAncestorWithClass(this,BundleTag.class);
            if (bundleTag != null) {
                _bundle = bundleTag.getBundle();
            }
        }
        if ( _bundle == null ) {
            _bundle = ResourceHelper.getBundle(pageContext);
        }
        return _bundle;
    }

    /**
     *  clears out the key for the next usage
     */
    public void release()
    {
        super.release();
        _key = null;
        _bundle = null;
    }

    /**
     *  must be overridden by a subclass to return a boolean value
     */
    public abstract boolean shouldEvaluate() throws JspException;

    /**
     *  returns (if any) the value specified for the key in the bundle.
     *
     *  if no value is specified, traps the MissingResourceException and returns null
     */
    protected String getValue() throws JspException
    {
        // ensure we have a key
        if ( _key == null) {
            throw new JspTagException(
                this._tagname + " tag, requires a key attribute.");
        }

        // ensure a bundle was defined
        ResourceBundle bundle = this.getBundle();
        if ( this.getBundle() == null) {
            throw new JspTagException(
                this._tagname + " tag, no bundle found for use with tag");
        }

        String value = null;
        try {
            value = bundle.getString(_key);
        } catch (java.util.MissingResourceException e) {
            value = null;
        }
        return value;
    }

    /**
     *  locates the bundle and tests whether the key has a value
     */
    public int doStartTag() throws JspException
    {
        int returnValue;

        // evaluate the body if a value was found, otherwise skip it
        if ( this.shouldEvaluate() ) {
            returnValue = BodyTag.EVAL_BODY_TAG;
        } else {
            returnValue = BodyTag.SKIP_BODY;
        }

        return returnValue;
    }

    /**
     * Prints the body of the tag.
     * <P>
     * Always return the flag for processing the rest of the JSP page.
     *
     * @return the flag for processing the rest of the JSP page.
     * @exception JspException if the writing out of the body fails.
     */
    public int doEndTag() throws JspException
    {
        try {
            if (bodyContent != null) {
                bodyContent.writeOut(bodyContent.getEnclosingWriter());
            }
        } catch (java.io.IOException ioe) {
            throw new JspTagException(
                this._tagname + " tag, IO Error: " + ioe.getMessage());
        }

        // This is not a loop, so always process the rest of the JSP page
        return BodyTag.EVAL_PAGE;
    }

}
