/*
 * $Header: /home/cvs/jakarta-taglibs/i18n/src/org/apache/taglibs/i18n/FormatNumberTag.java,v 1.4 2002/12/19 04:33:05 glenn Exp $
 * $Revision: 1.4 $
 * $Date: 2002/12/19 04:33:05 $
 *
 * ====================================================================
 * 
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:  
 *       "This product includes software developed by the 
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 */ 

package org.apache.taglibs.i18n;

import java.io.IOException;
import java.text.DecimalFormat;
import java.text.DecimalFormatSymbols;
import java.text.Format;
import java.text.NumberFormat;
import java.util.Enumeration;
import java.util.Locale;

/** Formats a {@link Number} instance using a {@link java.util.Locale} and the 
  * {@link NumberFormat} or a {@link DecimalFormat} if a pattern is specified.
  *
  * @author James Strachan
  * @version $Revision: 1.4 $
  */
public class FormatNumberTag extends FormatTagSupport {
    
    protected static final String _tagname = "i18n:formatNumber";

    private NumberFormat format;
    private String pattern;


    // Tag interface
    //-------------------------------------------------------------------------
    public void release() {
        super.release();
        format = null;
        pattern = null;
    }
    
    // Properties
    //-------------------------------------------------------------------------
    
    /** If no {@link NumberFormat} is configured then use a 
      * {@link DecimalFormat} instance if a pattern is configured else
      * use the default NumberFormat for the {@link java.util.Locale}
      */
    public Format getFormat() {
        if ( format == null ) {
            String pattern = getPattern();
            if ( pattern != null ) {
                format = getPatternFormat( pattern );
            }
            if ( pattern == null ) {
                format = getNumberFormat();
            }
        }
        return format;
    }

    public void setFormat( NumberFormat format ) {
        this.format = format;
    }

    public String getPattern() {
        return pattern;
    }
    
    public void setPattern( String pattern ) {
        this.pattern = pattern;
    }

    // Implementation methods
    //-------------------------------------------------------------------------
    protected NumberFormat getPatternFormat( String pattern ) {
        // XXXX: Use a Locale based cache for these objects as per Craig's 
        // XXXX: suggestion        
        return new DecimalFormat( pattern, new DecimalFormatSymbols( getLocale() ) );
    }
    
    /** @return the default number formatter for the current Locale 
      */
    protected NumberFormat getNumberFormat() {
        return NumberFormat.getInstance( getLocale() );
        // XXXX: what's the difference with the following:=
        // return NumberFormat.getNumberInstance( getLocale() );
    }
    
}
