/*
 * $Header: /home/cvs/jakarta-taglibs/i18n/src/org/apache/taglibs/i18n/FormatStringTag.java,v 1.3 2002/12/19 04:33:05 glenn Exp $
 * $Revision: 1.3 $
 * $Date: 2002/12/19 04:33:05 $
 *
 * ====================================================================
 * 
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:  
 *       "This product includes software developed by the 
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 */ 

package org.apache.taglibs.i18n;

import java.io.IOException;
import java.text.Format;
import java.text.DateFormat;
import java.util.Enumeration;
import java.util.Locale;

import javax.servlet.ServletRequest;
import javax.servlet.jsp.JspException;
import javax.servlet.jsp.JspWriter;
import javax.servlet.jsp.PageContext;
import javax.servlet.jsp.tagext.Tag;
import javax.servlet.jsp.tagext.TagSupport;

/** A simple tag that allows a String to be output with null handling.
  *
  * @author James Strachan
  * @version $Revision: 1.3 $
  */
public class FormatStringTag extends TagSupport {
    
    protected static final String _tagname = "i18n:formatString";

    /** the value to be formatted */
    private String value;    
    /** the text output if the value is null */
    private String defaultText = "";


    // Tag interface
    //-------------------------------------------------------------------------
    public int doStartTag() throws JspException {
        return EVAL_BODY_INCLUDE;
    }
    
    public int doEndTag() throws JspException {
        try {
            JspWriter out = pageContext.getOut();
            String text = getValue();
            if ( text == null ) {
                text = getDefaultText();
                if ( text == null ) {
                    text = "";
                }
            }
            out.print( text );
        }
        catch ( IOException e ) {
            handleIOException( e );
        }
        return EVAL_PAGE;
    }
    
    public void release() {
        super.release();
        value = null;
        defaultText = "";
    }
    
    // Properties
    //-------------------------------------------------------------------------    
    public String getValue() {
        return value;
    }
    
    public void setValue( String value ) {
        this.value = value;
    }

    public String getDefaultText() {
        return defaultText;
    }
    
    public void setDefaultText( String defaultText ) {
        this.defaultText = defaultText;
    }
    
    // Implementation methods
    //-------------------------------------------------------------------------    
    protected void handleIOException( IOException e ) throws JspException {
        pageContext.getServletContext().log( this._tagname + " tag, IOException: " + e );
        throw new JspException( this._tagname + " tag, IOException: " + e );
    }
}


