/*
 * $Header: /home/cvs/jakarta-taglibs/i18n/src/org/apache/taglibs/i18n/FormatTagSupport.java,v 1.6 2002/12/19 04:33:05 glenn Exp $
 * $Revision: 1.6 $
 * $Date: 2002/12/19 04:33:05 $
 *
 * ====================================================================
 * 
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:  
 *       "This product includes software developed by the 
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 */ 

package org.apache.taglibs.i18n;

import java.io.IOException;
import java.text.Format;
import java.text.DateFormat;
import java.util.Enumeration;
import java.util.Locale;

import javax.servlet.ServletRequest;
import javax.servlet.jsp.JspException;
import javax.servlet.jsp.JspTagException;
import javax.servlet.jsp.JspWriter;
import javax.servlet.jsp.PageContext;
import javax.servlet.jsp.tagext.Tag;
import javax.servlet.jsp.tagext.TagSupport;

/** An abstract base class for the formatting tags to provide 
  * implementation inheritence.
  *
  * @author James Strachan
  * @version $Revision: 1.6 $
  */
public abstract class FormatTagSupport extends TagSupport {
    
    protected static String _tagname = "i18n:";

    /** the value to be formatted */
    private Object value;    
    /** the locale used to format the value */
    private Locale locale;
    /** the text output if the value is null */
    private String defaultText = "";


    // Tag interface
    //-------------------------------------------------------------------------
    public int doStartTag() throws JspException {
        return EVAL_BODY_INCLUDE;
    }
    
    public int doEndTag() throws JspException {
        try {
            Object value = getValue();
            JspWriter out = pageContext.getOut();
            String text = null;
            if ( value != null ) {
                Format formatter = getFormat();
                if ( formatter == null ) {
                    throw new JspTagException(
                        this._tagname +
                        " tag, could not find valid Format instance");
                }
                text = formatter.format( value );
            }
            else {
                text = getDefaultText();
            }
            if ( text != null ) {
                out.print( text );
            }
        }
        catch ( IOException e ) {
            handleIOException( e );
        }
        return EVAL_PAGE;
    }
    
    public void release() {
        super.release();
        value = null;
        locale = null;
        defaultText = "";
    }
    
    // Properties
    //-------------------------------------------------------------------------    
    public Object getValue() {
        return value;
    }
    
    public void setValue( Object value ) {
        this.value = value;
    }

    /** If no {@link java.util.Locale} has been explicitly configured then use the
      * parent LocaleTag if present else the Locale from the ServletRequest
      * else use the default JVM {@link java.util.Locale}.
      */
    public Locale getLocale() {
        if ( locale == null ) {
            return findLocale();
        }
        return locale;
    }
    
    public void setLocale( Locale locale ) {
        this.locale = locale;
    }

    public String getDefaultText() {
        return defaultText;
    }
    
    public void setDefaultText( String defaultText ) {
        this.defaultText = defaultText;
    }

    
    // Implementation methods
    //-------------------------------------------------------------------------
    
    /** Abstract class to return the value formatter 
      */
    protected abstract Format getFormat();    
    
    protected void handleIOException( IOException e ) throws JspException {
        pageContext.getServletContext().log( "Caught: IOException: " + e );
        throw new JspTagException(
            this._tagname + " tag, IOException: " + e );
    }

    /** finds the current locale from either an outer LocaleTag or the 
      * current SerlvetRequest or the current JVM.
      *
      * @return a Locale instance
      */
    private Locale findLocale() {
        // lets try find a LocaleTag first
        LocaleTag localeTag = (LocaleTag) findAncestorWithClass( this, LocaleTag.class );
        if ( localeTag != null ) {
            return localeTag.getLocale();
        }
        else {
            return pageContext.getResponse().getLocale();
        }
    }
    
    
    /** A helper method for date, datetime & time based formatting tags.
      * This method converts a string into a DateFormat style code.
      */
    protected int getStyleCode( String style ) {
        if ( "short".equalsIgnoreCase( style ) ) {
            return DateFormat.SHORT; 
        }
        else if ( "medium".equalsIgnoreCase( style ) ) {
            return DateFormat.MEDIUM; 
        }
        else if ( "long".equalsIgnoreCase( style ) ) {
            return DateFormat.LONG; 
        }
        else if ( "full".equalsIgnoreCase( style ) ) {
            return DateFormat.FULL; 
        }
        else {
            return DateFormat.SHORT;
        }
    }
}


