/*
 * $Header: /home/cvs/jakarta-taglibs/i18n/src/org/apache/taglibs/i18n/LocaleTag.java,v 1.6 2002/12/19 04:33:05 glenn Exp $
 * $Revision: 1.6 $
 * $Date: 2002/12/19 04:33:05 $
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 */

package org.apache.taglibs.i18n;

import java.util.Enumeration;
import java.util.Locale;

import javax.servlet.ServletRequest;
import javax.servlet.jsp.JspException;
import javax.servlet.jsp.PageContext;
import javax.servlet.jsp.tagext.TagSupport;


/** This tag defines a {@link java.util.Locale} context for use by other
  * inner JSP tags.
  *
  * If no locale has been configured directly via the "locale" property
  * then the language, country and optional varient properties are used
  * to create a new Locale instance.
  * If these properties are not specified then the Locale is taken from
  * {@link javax.servlet.ServletRequest} is used.
  * If still no {@link java.util.Locale} could be found then the default JVM
  * {@link java.util.Locale} is used.
  *
  * @author James Strachan
  * @version $Revision: 1.6 $
  */
public class LocaleTag extends TagSupport
{
    /** Holds value of property locale. */
    private Locale locale;
    /** Holds value of property localeRef. */
    private String localeRef;
    /** Holds value of property language. */
    private String language;
    /** Holds value of property country. */
    private String country;
    /** Holds value of property variant. */
    private String variant;
    /** specifies whether or not the response locale should be changed to match
     *  the locale used by this tag */
    private boolean changeResponseLocale = true;


    // Tag interface
    //-------------------------------------------------------------------------
    public int doStartTag() throws JspException
    {
        // set the locale as a variable in the page
        if ( this.getId() != null ) {
            pageContext.setAttribute(getId(),getLocale());
        }
    
    return EVAL_BODY_INCLUDE;
    }

    /**
     *  Sets the response locale if the changeResponseLocale attribute was set
     *  to true, OR if changeResponseLocale was unset and the tag was empty
     */
    public int doEndTag() throws JspException
    {
        if (this.changeResponseLocale) {
            // set the locale for the response
            pageContext.getResponse().setLocale(getLocale());
        }
        
    return EVAL_PAGE;
    }

    public void setChangeResponseLocale(boolean value)
    {
        changeResponseLocale = value;
    }

    public void release()
    {
        super.release();
        locale = null;
        language = null;
        country = null;
        variant = null;
        changeResponseLocale = true;
    }

    // Properties
    //-------------------------------------------------------------------------
    protected final Locale getLocale()
    {
        if (localeRef != null) {
            locale = (Locale)pageContext.findAttribute(localeRef);
        }
        if (locale == null) {
            locale = createLocale();
        }
        return locale;
    }

    public final void setLocale(Locale locale)
    {
        this.locale = locale;
    }

    /**
     *  Provides a key to retrieve a locale via findAttribute()
     */
    public final void setLocaleRef(String value)
    {
        this.localeRef = value;
    }

    public final void setLanguage(String language)
    {
        this.language = language;
    }

    public final void setCountry(String country)
    {
        this.country = country;
    }

    public final void setVariant(String variant)
    {
        this.variant = variant;
    }

    // Implementation methods
    //-------------------------------------------------------------------------
    private Locale createLocale()
    {
        // let's try use the language & country properties first...
        Locale locale = null;
        if (language == null) {
            locale = pageContext.getResponse().getLocale();
        } else if (country == null) {
            locale = new Locale(language,"");
        } else if (variant == null) {
            locale = new Locale(language, country);
        } else {
            locale = new Locale(language, country, variant);
        }

        return locale;
    }

}
