/*
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:  
 *       "This product includes software developed by the 
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 */ 
package org.apache.taglibs.input;

import java.util.*;
import javax.servlet.http.*;
import javax.servlet.jsp.*;
import javax.servlet.jsp.tagext.*;

/**
 *
 *  This class implements the &lt;input:form&gt; tag, which presents an 
 *  &lt;form ... /&gt; element.
 *
 *  @version 0.90
 *  @author Shawn Bayern
 *  @author Lance Lavandowska
 *  @author Karl von Randow
 */

public class Form extends TagSupport {

    private String name;        // name of the form
    private String action;      // form action
    private Map attributes;     // attributes of the <input> element
    private String attributesText; // attributes of the <input> element as text
    private String beanId;      // bean id to get default values from
    private String method;      // form method
    private String encType;     // form encType
    
    public int doStartTag() throws JspException {
        try {
            // get what we need from the page
            JspWriter out = pageContext.getOut();

            // start building up the tag
            out.print("<form method=\"" );
            out.print( method != null ? method : "GET" );
            out.print( "\" action=\"" );
            if ( action != null ) {
                if ( action.length() > 0 && pageContext.getResponse() instanceof HttpServletResponse ) {
                    out.print( ((HttpServletResponse)pageContext.getResponse()).encodeURL( action ) );
                }
                else {
                    out.print( action );
                }
            }
            out.print( "\" " );
            if ( name != null ) {
                out.print( "name=\"" + Util.quote(name) + "\" ");
            }
            if ( encType != null ) {
                out.print( "encType=\"" + encType + "\" " );
            }

            // include any attributes we've got here
            Util.printAttributes(out, attributes);
            if ( attributesText != null ) {
                out.print( attributesText + " " );
            }

            // end the tag
            out.print(">");

        }
        catch (Exception ex) {
            throw new JspTagException(ex.getMessage());
        }
        return EVAL_BODY_INCLUDE;
    }
    
    public int doEndTag() throws JspException {
        try {
            JspWriter out = pageContext.getOut();
            out.print( "</form>" );
        }
        catch (Exception ex) {
            throw new JspTagException(ex.getMessage());
        }
        return EVAL_PAGE;
    }
    
    public void setName(String x) {
        name = x;
    }

    public void setAttributes(Map x) {
        attributes = x;
    }
    
    public void setAttributesText( String x ) {
        attributesText = x;
    }
    
    public void setBean( String x ) {
        beanId = x;
    }
    
    public String getBean() {
        return beanId;
    }

    public void setAction(String x) {
        action = x;
    }

    public void setMethod(String x) {
        method = x;
    }

    public void setEncType(String x) {
        encType = x;
    }
    
    /** Getter for property name.
     * @return Value of property name.
     */
    public String getName() { return name; }
    
    /** Getter for property method.
     * @return Value of property method.
     */
    public String getMethod() { return method; }
    
    /** Getter for property encType.
     * @return Value of property encType.
     */
    public String getEncType() { return encType; }
    
    /** Getter for property attributesText.
     * @return Value of property attributesText.
     */
    public String getAttributesText() { return attributesText; }
    
    /** Getter for property attributes.
     * @return Value of property attributes.
     */
    public Map getAttributes() { return attributes; }
    
    /** Getter for property action.
     * @return Value of property action.
     */
    public String getAction() { return action; }
    
}
