/*
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:  
 *       "This product includes software developed by the 
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 */ 
package org.apache.taglibs.input;

import java.util.*;
import javax.servlet.jsp.*;
import javax.servlet.jsp.tagext.*;

/**
 *
 *  This class implements the &lt;input:option&gt; tag, which presents an
 *  &lt;option&gt; form element.
 *
 *  @version 0.90
 *  @author Karl von Randow
 */

public class Option extends TagSupport {

    private String value;       // value of option
    private Map attributes;     // attributes of the <select> element
    private String attributesText; // attributes of the <option> element as text

    public int doStartTag() throws JspException {
        try {
            // get what we need from the page
            JspWriter out = pageContext.getOut();

            // start building up the tag
            out.print("<option " );
            if ( value != null ) {
                out.print( "value=\"" + Util.quote(value) + "\" ");
            }

            // include any attributes we've got here
            Util.printAttributes(out, attributes);
            if ( attributesText != null ) {
                out.print( attributesText + " " );
            }

            /*
             * Print out our options, selecting one or more if appropriate.
             * If there are multiple selections but the page doesn't call
             * for a <select> that accepts them, ignore the selections.
             * This is preferable to throwing a JspException because the
             * (end) user can control input, and we don't want the user
             * causing exceptions in our application.
             */

            HashMap chosen = getChosen();
            
            if ( value != null && chosen != null && chosen.containsKey( value ) ) {
                out.print( "selected=\"true\" " );
            }

            // end the starting tag
            out.print(">");

        } catch (Exception ex) {
            throw new JspTagException(ex.getMessage());
        }
        return EVAL_BODY_INCLUDE;
    }
    
    public int doEndTag() throws JspException {
        try {
            JspWriter out = pageContext.getOut();
            out.print( "</option>" );
            return EVAL_PAGE;
        } catch (Exception ex) {
            throw new JspTagException(ex.getMessage());
        }
    }
    
    protected HashMap getChosen() throws JspException {
        Select selectTag = (Select) findAncestorWithClass( this, Select.class );
        if ( selectTag != null ) {
            return selectTag.getChosen();
        }
        else {
            throw new JspTagException( "option tag used outside a select tag" );
        }
    }

    public void setValue(String x) {
        value = x;
    }
    
    /** Getter for property value.
     * @return Value of property value.
     */
    public String getValue() { return value; }

    public void setAttributes(Map x) {
        attributes = x;
    }
    
    /** Getter for property attributes.
     * @return Value of property attributes.
     */
    public Map getAttributes() { return attributes; }
    
    public void setAttributesText( String x ) {
        attributesText = x;
    }
    
    /** Getter for property attributesText.
     * @return Value of property attributesText.
     */
    public String getAttributesText() { return attributesText; }
    
}
