/*
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 */

package org.apache.taglibs.jms;

import javax.jms.Destination;
import javax.jms.JMSException;
import javax.jms.Message;
import javax.servlet.jsp.JspException;

import org.apache.commons.messenger.Messenger;

/** Creates a JMS message
  *
  * @author <a href="mailto:jstrachan@apache.org">James Strachan</a>
  * @version $Revision: 1.1 $
  */
public class MessageTag extends AbstractTag {

    /** The name of the Message variable that is created */
    private String var;    
    
    /** The JMS Message created */
    private Message message;
    
    /** The Messenger used to access the JMS connection */
    private Messenger connection;
    
    public MessageTag() {
    }
    
    /** Adds a JMS property to the message */
    public void addProperty(String name, Object value) throws JspException, JMSException {
        Message message = getMessage();
        message.setObjectProperty(name, value);
    }
    
    // BodyTag interface
    //-------------------------------------------------------------------------                    
    public int doStartTag() throws JspException {
        return EVAL_BODY_INCLUDE;
    }
    
    public int doEndTag() throws JspException {
        try {
            if ( var == null ) {
                // expose message to parent message consumer
                SendTag tag = (SendTag) findAncestorWithClass( this, SendTag.class );
                if ( tag == null ) {
                    throw new JspException("<jms:message> tags must either have the 'var' attribute specified or be used inside a <jms:send> tag");
                }
                tag.setMessage( getMessage() );            
            }
            else {
                pageContext.setAttribute( var, getMessage() );
            }
        }
        catch (JMSException e) {
            throw new JspException( "Failed to create message: " + e, e );
        }
        return EVAL_PAGE;
    }
    
    public void release() {
        super.release();
        var = null;
        connection = null;
        message = null;
    }
    
    
    // Properties
    //-------------------------------------------------------------------------                            
    
    /** Sets the name of the variable that the message will be exported to */
    public void setVar(String var) {
        this.var = var;        
    }
    
    public Messenger getConnection() throws JspException, JMSException {
        if ( connection == null ) {
            return findConnection();
        }
        return connection;
    }
    
    public void setConnection(Messenger connection) {
        this.connection = connection;
    }
    
    public Message getMessage() throws JspException, JMSException {
        if ( message == null ) {
            message = createMessage();
        }
        return message;
    }
    

    // JMS related properties
    
    public void setCorrelationID(String correlationID) throws JspException, JMSException {
        getMessage().setJMSCorrelationID(correlationID);
    }
    
    public void setReplyTo(Destination destination) throws JspException, JMSException {
        getMessage().setJMSReplyTo(destination);
    }
    
    public void setType(String type) throws JspException, JMSException {
        getMessage().setJMSType(type);
    }
    
    // Implementation methods
    //-------------------------------------------------------------------------                            
    protected Messenger findConnection() throws JspException, JMSException {
        ConnectionContext messengerTag = (ConnectionContext) findAncestorWithClass( this, ConnectionContext.class );
        if ( messengerTag == null ) {
            throw new JspException("This tag must be within a <jms:connection> tag or the 'connection' attribute should be specified");
        }
        return messengerTag.getConnection();
    }
    
    protected Message createMessage() throws JspException, JMSException {
        return getConnection().createMessage();
    }    
}    
