/*
 * $Header: /home/cvs/jakarta-taglibs/mailer/src/org/apache/taglibs/mailer/AddRecipientTag.java,v 1.7 2002/12/03 16:08:19 glenn Exp $
 * $Revision: 1.7 $
 * $Date: 2002/12/03 16:08:19 $
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 */

package org.apache.taglibs.mailer;

import java.util.*;
import javax.servlet.jsp.*;
import javax.servlet.jsp.tagext.*;

/**
 * AddRecipientTag - JSP tag <b>addrecipient</b> is used to add any type of 
 *                recipient to an already existant list of recipients in an 
 *                e-mail message. Two attributes are required, type and address.
 *                Type may be either "to", "cc", or "bcc" and address should be 
 *                a string representation of the recipients e-mail address.
 *
 * <tag>
 *        <name>addrecipient</name>
 *	  <tagclass>org.apache.taglibs.mailer.AddRecipientTag</tagclass>
 *	  <bodycontent>JSP</bodycontent>
 *	  <info>
 *          Append a recipient to the current recipients of the e-mail.
 *        </info>
 *
 *	  <attribute>
 *	     <name>type</name>
 *	     <required>true</required>
 *	    <rtexprvalue>false</rtexprvalue>
 *	  </attribute>
 *	  <attribute>
 *	     <name>address</name>
 *	     <required>false</required>
 *	    <rtexprvalue>false</rtexprvalue>
 *	  </attribute>
 * </tag>
 *
 * @author Rich Catlett Jayson Falkner
 *
 * @version 1.0
 *
 */

public class AddRecipientTag extends BodyTagSupport {

    /**
     * The type of address to add either to, cc, or bcc
     */
    private String type_string = null;
    private int type = 0;

    /**
     * The address to be added
     */
    private String address = null;

    /**
     *  implementation of the method from the tag interface that tells the JSP
     *  page what to do at the start of this tag
     *
     *  @throws JSPTagException  thrown when an error occurs while processing the
     *                        body of this method
     *
     *  @return SKIP_BODY  int telling the tag handler to not evaluate the body
     *                     of this tag
     *          EVAL_BODY_TAG  int telling the tag handler to evaluate the body
     *                         of this tag
     *
     */
    public final int doStartTag() throws JspException {
        if (address != null && address.length() > 0 ) {
            addToParent(address);
            return SKIP_BODY;
	}
	return EVAL_BODY_TAG;
    }


    /**
     *  implementation of the method from the tag interface that tells the JSP
     *  page what to do after the body of this tag
     *
     *  @throws JSPException  thrown when an error occurs while processing the
     *                        body of this method
     *
     *  @return SKIP_BODY  int telling the tag handler to not evaluate the body
     *                     of this tag again
     *
     */
    public final int doAfterBody() throws JspException {
        BodyContent body = getBodyContent();
        String addr = body.getString();
        // Clear the body since we only used it as input for the email address
        body.clearBody();
        if (addr != null && addr.length() > 0 ) {
            addToParent(addr);
            return SKIP_BODY;
	} else {
	    throw new JspException("addrecpient tag could not find an email address. set " +
			     " the address attribute, or place the address in" +
			     " the body of the tag.");
        }
    }

    /**
     * set the type of recipient for the address
     *
     * @param type  string that is the type of the address either 
     *   "to", "cc", or "bcc".
     */
    public final void setType(String type) throws JspTagException {
      this.type_string = type.trim();
      if (type_string.equalsIgnoreCase(MailTag.TO_ADDRESS_STRING)) {
          this.type = MailTag.TO_ADDRESS;
      } else if (type_string.equalsIgnoreCase(MailTag.CC_ADDRESS_STRING)) {
          this.type = MailTag.CC_ADDRESS;
      } else if (type_string.equalsIgnoreCase(MailTag.BCC_ADDRESS_STRING)) {
          this.type = MailTag.BCC_ADDRESS;
      } else {
          throw new JspTagException
             ("addrecipient tag type attribute must be \"to\", \"cc\", or \"bcc\"");
      }
    }

    /**
     * set the value for an address to be later added to the email
     *
     * @param address  string that is an address to be added to the 
     *   "to", "cc", or "bcc" lists of addresses.
     */
    public final void setAddress(String address) {
      this.address = address.trim();
    }

    private void addToParent(String addr) throws JspTagException {
      // parent tag must be a MailTag, gives access to methods in parent
      MailTag myparent = 
          (MailTag)findAncestorWithClass
              (this, MailTag.class);

      if (myparent == null) {
          throw new JspTagException("addrecipient tag not nested within mail tag");
      }

      // Make sure type is set..either "to", "cc", or "bcc"
      switch (type) {
          case MailTag.TO_ADDRESS:
              // set to in the parent tag
              myparent.addTo(addr);
              break;
          case MailTag.CC_ADDRESS:
              // set cc in the parent tag
              myparent.addCc(addr);
              break;
          case MailTag.BCC_ADDRESS:
              // set bcc in the parent tag
              myparent.addBcc(addr);
              break;
          default:
              throw new JspTagException("addrecipient tag type attribute is not set. " +
                           " Specify either \"to\", \"cc\", or \"bcc\".");
      }
    }

}
