/*
 * $Header: /home/cvs/jakarta-taglibs/mailer/src/org/apache/taglibs/mailer/ErrorTag.java,v 1.7 2002/12/03 16:08:19 glenn Exp $
 * $Revision: 1.7 $
 * $Date: 2002/12/03 16:08:19 $
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 */

package org.apache.taglibs.mailer;

import java.util.*;
import java.io.IOException;
import javax.servlet.jsp.*;
import javax.servlet.jsp.tagext.*;

/**
 * ErrorTag - JSP tag <b>Error</b> is used to get the error message when creating
 *            an error page.
 *
 * <tag>
 *        <name>error</name>
 *	  <tagclass>org.apache.taglibs.mail.ErrorTag</tagclass>
 *	  <bodycontent>JSP</bodycontent>
 *	  <info>Get the error message if mail was unable to be sent</info>
 *
 *        <attribute>
 *                <name>id</name>
 *                <required>true</required>
 *                <rtexprvalue>false</rtexprvalue>
 *        </attribute>
 * </tag>
 *
 * @author Rich Catlett
 *
 * @version 1.0
 *
 */

public class ErrorTag extends BodyTagSupport {

    // an iterator for the list of error messages
    private Iterator errorlist = null;
    private ArrayList  errors = null;  // the error message
    private String error = null;      // the current error from the list

    /**
     *  implementation of the method from the tag interface that tells the JSP
     *  page what to do when the start tag is encountered
     *
     *  @throws JSPException  thrown when an error occurs while processing the
     *                        body of this method
     *
     *  @return - SKIP_BODY if no errors exist, EVAL_BODY_TAG if errors do exist
     *
     */
    public final int doStartTag() throws JspException {

	// parent tag must be a MailTag, gives access to methods in parent
	SendTag myparent = (SendTag)findAncestorWithClass(this, SendTag.class);

	if (myparent == null)
	    throw new JspException("error tag not nested within send tag");
        else
	    if ((errors = myparent.getError()) == null)
		return SKIP_BODY;

	errorlist = errors.iterator();
	if (!errorlist.hasNext())
	    return SKIP_BODY;

	// get the next element in the list
	error = (String)errorlist.next();

	// final check to make sure property exists
	if (error == null)
	    return SKIP_BODY;

	pageContext.setAttribute(id, this, PageContext.PAGE_SCOPE);
	// taglibs 1.1
	return EVAL_BODY_TAG;
	// taglibs 1.2
	//return EVAL_BODY_BUFFERED;
    }

    /**
     * Implementation of the method from the tag interface that gets called at 
     * end of each error tag.
     *
     * @return EVAL_BODY_TAG if there is another error, or SKIP_BODY if there
     *                      are no more errors 
     */
    public final int doAfterBody() throws JspException
    {
	// See if this is the last or a named parameter
	if (!errorlist.hasNext())
	    return SKIP_BODY;
	// There is another parameter, so loop again
	error = (String)errorlist.next();
	if (error == null)
	    return SKIP_BODY;
	// taglibs 1.1
	return EVAL_BODY_TAG;
	// taglibs 1.2
	// return EVAL_BODY_AGAIN;
    }

    /**
     *  implementation of the method from the tag interface that tells the JSP
     *  page what to do when the start tag is encountered
     *
     *  @throws JSPException  thrown when an error occurs while processing the
     *                        body of this method
     *
     *  @return - EVAL_PAGE
     *
     */
    public final int doEndTag() throws JspException {
	try {
	    if (bodyContent != null)
		bodyContent.writeOut(bodyContent.getEnclosingWriter());
	} catch (java.io.IOException e) {
	    throw new JspException("IO Error: " + e.getMessage());
	}
	return EVAL_PAGE;
    }

    /**
     * Returns the name of the current error.
     * <p>
     * &lt;jsp:getProperty name=<i>"id"</i> property="error"/&gt;
     *
     * @return String - the current error in the list
     */
    public final String getError()
    {
	return error;
    }

    /**
     * Remove the script variable after error tag is closed out
     */
    public final void release()
    {
	if( id != null && id.length() > 0 )
	    pageContext.removeAttribute(id,PageContext.PAGE_SCOPE);
    }
}
