/*
 * $Header: /home/cvs/jakarta-taglibs/random/src/org/apache/taglibs/random/RandomNum.java,v 1.2 2001/08/14 18:32:17 catlett Exp $
 * $Revision: 1.2 $
 * $Date: 2001/08/14 18:32:17 $
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 */

package org.apache.taglibs.random;

import java.util.*;
import java.security.SecureRandom;
import java.security.NoSuchAlgorithmException;
import java.security.NoSuchProviderException;
import javax.servlet.jsp.JspException;

/**
 * The RandomNum will produce a variable set of random numbers.
 *
 * @author Rich Catlett
 *
 * @version 1.0
 *
 */

public class RandomNum {

    /**
     * the random number is stroed here for retrieval
     */
    private Long randomnum = null;
    /**
     * the random number is stroed here for retrieval
     */
    private Float randomfloat = null;
    /**
     * flag tells if a float value is expected or not
     */
    private boolean floatvalue = false;
    /**
     * upper bound in which to search for a number defaults to 9
     */
    private long upper = 100;
    /**
     * lower bound in which to search for a number defaults to 0
     */
    private long lower = 0;
    /**
     * the algorithm to use for a SecureRandom object
     */
    private String algorithm = null;
    /**
     * the provider package to check for the algorithm
     */
    private String provider = null;
    /**
     * boolean value that marks if a Random or SecureRandom object is to be used,
     * default value of false says that a Random object will be used
     */
    private boolean secure = false;
    /**
     * random object that could be used
     */
    private Random random = null;
    /**
     * SecureRandom object that could be used
     */
    private SecureRandom secrandom = null;

    /**
     * nethod determines if a Random or SecureRandom object is to be used to
     * generate the random number
     *
     */
    private final float getFloat() {
	if (random == null)
	    return secrandom.nextFloat();
	else
	    return random.nextFloat();
    }

    /**
     * generate the Random object that will be used for this random number 
     * generator
     *
     */
    public final void generateRandomObject() throws JspException {

	// check to see if the object is a SecureRandom object
	if (secure) {
	    try {
		// get an instance of a SecureRandom object
		if (provider != null)
		    // search for algorithm in package provider
		    secrandom = SecureRandom.getInstance(algorithm, provider);
		else
		    secrandom = SecureRandom.getInstance(algorithm);
	    } catch (NoSuchAlgorithmException ne) {
		throw new JspException(ne.getMessage());
	    } catch (NoSuchProviderException pe) {
		throw new JspException(pe.getMessage());
	    }
        } else
	    random = new Random();
    }

    /**
     * generate the random number
     *
     */
    private final void generaterandom() {

	int tmprandom = 0;  // temp storage for random generated number
	Integer rand;

	// check to see if float value is expected
	if (floatvalue)
	    randomfloat = new Float(getFloat());
        else
	    randomnum = new Long(lower + (long) ((getFloat() * (upper - lower))));
    }

    /**
     * get the random number, for use with the 
     * &lt;jsp:getProperty name=<i>"id"</i> property="floatval"/&gt;
     *
     * @return - randomly created float value
     *
     * @throws - JspException if the range entered for the number generator is not
     *           0-1
     */
    public final Number getRandom() {

	generaterandom(); // generate the first random number

	if (floatvalue)
	    return randomfloat;
	else
	    return randomnum;
    }

    /**
     * set the Range
     *
     * @param low  lower bound of the range
     * @param up  upper bound of the range
     *
     */
    public final void setRange(long low, long up) {

	// set the upper and lower bound of the range
	lower = low;
	upper = up;

	// check to see if a float value is expected
	if ((lower == 0) && (upper == 1))
	    floatvalue = true;
    }

    /**
     * set the algorithm name
     *
     * @param value  name of the algorithm to use for a SecureRandom object
     *
     */
    public final void setAlgorithm(String value) {
	algorithm = value;
	secure = true;  // a SecureRandom object is to be used
    }

    /**
     * set the provider name
     *
     * @param value  name of the package to check for the algorithm
     *
     */
    public final void setProvider(String value) {
	provider = value;
    }
}
