/*
 * $Header: /home/cvs/jakarta-taglibs/regexp/src/org/apache/taglibs/regexp/MatchTag.java,v 1.4 2001/07/06 02:18:44 glenn Exp $
 * $Revision: 1.4 $
 * $Date: 2001/07/06 02:18:44 $
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 */

package org.apache.taglibs.regexp;

import java.util.*;
import org.apache.oro.text.*;
import org.apache.oro.text.perl.*;
import org.apache.oro.text.regex.*;
import javax.servlet.*;
import javax.servlet.http.*;
import javax.servlet.jsp.*;
import javax.servlet.jsp.tagext.*;

/**
 * JSP Tag <b>match</b>, used to implement a perl style
 * match on the text.
 * <p>
 * The body of the tag iterates through each match found.
 * The normal &lt;jsp:getProperty/&gt; can be used to get
 * the match results using the id of the <b>match</b> tag
 * script variable.
 * <p>
 * The attribute <b>text</b> must be set to the id of a
 * <b>text</b> tag, and the <b>regexp</b> attribute must
 * be set to the id of a <b>regexp</b> tag.
 * <p>
 * JSP Tag Lib Descriptor
 * <p><pre>
 * &lt;name&gt;match&lt;/name&gt;
 * &lt;tagclass&gt;org.apache.taglibs.regexp.MatchTag&lt;/tagclass&gt;
 * &lt;bodycontent&gt;JSP&lt;/bodycontent&gt;
 * &lt;info&gt;Uses the regexp to find matches in text, loops for each match found.&lt;/info&gt;
 *   &lt;attribute&gt;
 *     &lt;name&gt;id&lt;/name&gt;
 *     &lt;required&gt;true&lt;/required&gt;
 *     &lt;rtexprvalue&gt;false&lt;/rtexprvalue&gt;
 *   &lt;/attribute&gt;
 *   &lt;attribute&gt;
 *     &lt;name&gt;text&lt;/name&gt;
 *     &lt;required&gt;true&lt;/required&gt;
 *     &lt;rtexprvalue&gt;false&lt;/rtexprvalue&gt;
 *   &lt;/attribute&gt;
 *   &lt;attribute&gt;
 *     &lt;name&gt;regexp&lt;/name&gt;
 *     &lt;required&gt;true&lt;/required&gt;
 *     &lt;rtexprvalue&gt;false&lt;/rtexprvalue&gt;
 *   &lt;/attribute&gt;
 * </pre>
 *
 * @see RegexpTag
 * @see TextTag
 * @see RegexpData
 * @see TextData
 *
 * @author Glenn Nielsen
 */

public class MatchTag extends BodyTagSupport
{
    private String regexpid = null;
    private String textid = null;
    private Perl5Util perl = new Perl5Util(RegexpData.getPatternCache());
    private PatternMatcherInput pmi = null;
    private RegexpData rd = null;

    /**
     * Setup to loop through matches found in the text using regexp
     *
     * @return EVAL_BODY_TAG if a match is found, BODY_SKIP if no matches are found
     */
    public final int doStartTag() throws JspException
    {
	TextData td = (TextData)pageContext.getAttribute(textid,PageContext.PAGE_SCOPE);
	if( td == null )
	    throw new JspException(
		"regexp tag match could not find text tag with id: " +
		textid);
	pmi = new PatternMatcherInput(td.getText());

        rd = (RegexpData)pageContext.getAttribute(regexpid,PageContext.PAGE_SCOPE);
        if( rd == null )
            throw new JspException(
                "regexp tag match could not find regexp tag with id: " +
                regexpid);
	if( !perl.match(rd.getRegexp(),pmi) )
	    return SKIP_BODY;

	pageContext.setAttribute(id,this,PageContext.PAGE_SCOPE);

	return EVAL_BODY_TAG;
    }

    /** 
     * Method called at end of each iteration of the match tag
     *
     * @return EVAL_BODY_TAG if there is another string match, or BODY_SKIP if there are no more strings
     */
    public final int doAfterBody() throws JspException
    {
	if( !perl.match(rd.getRegexp(),pmi) )
	    return SKIP_BODY;
	return EVAL_BODY_TAG;
    }

    /**
     * Method called at end of match Tag
     *
     * @return EVAL_PAGE
     */
    public final int doEndTag() throws JspException
    {
        if( id != null && id.length() > 0 )
            pageContext.removeAttribute(id,PageContext.PAGE_SCOPE);
	try {
	    if(bodyContent != null)
		bodyContent.writeOut(bodyContent.getEnclosingWriter());
	} catch(java.io.IOException e) {
	    throw new JspException("IO Error: " + e.getMessage());
	}
	return EVAL_PAGE;
    }

    /*
     * Set the required attribute <b>regexp</b>
     *
     * @param String name of regexp script variable
     */
    public final void setRegexp(String str)
    {   
	regexpid = str;
    }

    /*
     * Set the required attribute <b>text</b>
     *
     * @param String name of text script variable
     */
    public final void setText(String str)
    {               
        textid = str;
    }

    /**
     * Returns the entire string that was matched on
     * using &lt;jsp:getProperty name=<i>"id"</i> property="match"/&gt;
     * 
     * @return String - entire text of match
     */
    public final String getMatch() 
    {
        return perl.toString();
    }

    /**
     * Returns the string preceding the current match
     * using &lt;jsp:getProperty name=<i>"id"</i> property="preMatch"/&gt;
     *
     * @return String - text preceding match
     */
    public final String getPreMatch()
    {
	return perl.preMatch();
    }

    /**
     * Returns the string after the current match                                              
     * using &lt;jsp:getProperty name=<i>"id"</i> property="postMatch"/&gt;
     * 
     * @return String - text after the match
     */
    public final String getPostMatch() 
    {
        return perl.postMatch();
    }

    /**
     * Returns the string for the group parenthesized number
     *
     * @return String - for group parenthesized number
     */
    public final String getGroup(int i)
    {
	String group = perl.group(i);
	if( group == null )
	    return "";
	return group;
    }

}
