/*
 * $Header: /home/cvs/jakarta-taglibs/request/src/org/apache/taglibs/request/CookiesTag.java,v 1.4 2001/07/13 00:08:58 glenn Exp $
 * $Revision: 1.4 $
 * $Date: 2001/07/13 00:08:58 $
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 */

package org.apache.taglibs.request;

import javax.servlet.*;
import javax.servlet.http.*;
import javax.servlet.jsp.*;
import javax.servlet.jsp.tagext.*;

/**
 * JSP Tag <b>cookies</b>, used to get HttpServletRequest cookie information
 * using the standard JSP &lt;jsp:getProperty&gt; tag.
 * <p>
 * The script variable of name <b>id</b> is availble only within the
 * body of the <b>cookies</b> tag.
 * <p>
 * Loops through all the cookies received by the HttpServletRequest.
 * <p>
 * If the optional attribute <b>name</b> is present only the cookie of
 * that name is retreived.
 * <p>
 * JSP Tag Lib Descriptor
 * <p><pre>
 * &lt;name&gt;cookies&lt;/name&gt;
 * &lt;tagclass&gt;org.apache.taglibs.request.CookiesTag&lt;/tagclass&gt;
 * &lt;teiclass&gt;org.apache.taglibs.request.CookiesTEI&lt;/teiclass&gt;
 * &lt;bodycontent&gt;JSP&lt;/bodycontent&gt;
 * &lt;info&gt;Loop through all cookies or get a single cookie.&lt;/info&gt;
 *   &lt;attribute&gt;
 *     &lt;name&gt;id&lt;/name&gt;
 *     &lt;required&gt;true&lt;/required&gt;
 *     &lt;rtexprvalue&gt;false&lt;/rtexprvalue&gt;
 *   &lt;/attribute&gt;
 *   &lt;attribute&gt;
 *     &lt;name&gt;name&lt;/name&gt;
 *     &lt;required&gt;false&lt;/required&gt;
 *     &lt;rtexprvalue&gt;false&lt;/rtexprvalue&gt;
 *   &lt;/attribute&gt;
 * </pre>
 *
 * @author Glenn Nielsen
 */

public class CookiesTag extends BodyTagSupport
{
    // All the cookies received with request
    private Cookie [] cookies = null;
    // Current cookie
    private Cookie cookie = null;
    // Number of cookies
    private int cookie_num = 0;
    // Optional attribute, name of single cookie to get
    private String name = null;

    /**
     * Gets the cookies that came with the request or cookie with <b>name</b>.
     *
     * @return SKIP_BODY if no cookies or cookie with <b>name</b> is not found, EVAL_BODY_TAG if cookie exists
     */
    public final int doStartTag() throws JspException
    {
	// Get the cookies
	cookies = ((HttpServletRequest)pageContext.getRequest()).getCookies();
	if( cookies == null || cookies.length == 0 )
	    return SKIP_BODY;

	if( name != null ) {
	    // If interested in only one cookie, find it
	    for( int i = 0; i < cookies.length; i++ ) {
		if( cookies[i].getName().equals(name) ) {
		    cookie = cookies[i];
		    break;
		}
	    }
	    if( cookie == null ) // Cookie wasn't found
		return SKIP_BODY;
	} else {
	    cookie = cookies[0];
	}
	pageContext.setAttribute(id,this,PageContext.PAGE_SCOPE);
	return EVAL_BODY_TAG;
    }

    /**
     * Method called at end of each cookies tag.
     *
     * @return EVAL_BODY_TAG if there is another cookie, or SKIP_BODY if there are no more cookies or this is a named cookie
     */
    public final int doAfterBody() throws JspException
    {
	// increment the cookie number from the cookies array
	cookie_num++;
	// See if this is the last or a named cookie
	if( name != null || cookies == null || cookie_num >= cookies.length )
	    return SKIP_BODY;
	// There is another cookie, so loop again
	cookie = cookies[cookie_num];
	if( cookie == null )
	    return SKIP_BODY;
	return EVAL_BODY_TAG;
    }

    /**
     * Method called at end of Tag
     *
     * @return EVAL_PAGE
     */
    public final int doEndTag() throws JspException
    {
        pageContext.removeAttribute(id,PageContext.PAGE_SCOPE);
	try
	{
	    if(bodyContent != null)
	    bodyContent.writeOut(bodyContent.getEnclosingWriter());
	} catch(java.io.IOException e)
	{
	    throw new JspException("IO Error: " + e.getMessage());
	}
	return EVAL_PAGE;
    }

    /**
     * Set the optional tag attribute <b>name</b>.
     *
     * @param String name of cookie                            
     */
    public final void setName(String str)
    {
	name = str;
    }

    /**
     * Returns the comment describing the purpose of this cookie, or ""
     * if the cookie has no comment.
     * <p>
     * &lt;jsp:getProperty name=<i>"id"</i> property="comment"/&gt;
     *
     * @return String - cookie comment
     */
    public final String getComment()
    {
	return cookie.getComment();
    }

    /**
     * Returns the domain name set for this cookie.
     * <p>
     * &lt;jsp:getProperty name=<i>"id"</i> property="domain"/&gt;
     *
     * @return String - cookie domain
     */
    public final String getDomain()
    {
	return cookie.getDomain();
    }

    /**
     * Returns the maximum age of the cookie, specified in seconds,
     * By default, -1 indicating the cookie will persist until browser
     * shutdown.
     * <p>
     * &lt;jsp:getProperty name=<i>"id"</i> property="maxAge"/&gt;
     *
     * @return String - cookie max age
     */
    public final String getMaxAge()
    {
	return "" + cookie.getMaxAge();
    }

    /**
     * Returns the name of the cookie.
     * <p>
     * &lt;jsp:getProperty name=<i>"id"</i> property="name"/&gt;
     *
     * @return String - cookie name
     */
    public final String getName()
    {
	return cookie.getName();
    }

    /**
     * Returns the path on the server to which the browser returns this cookie.
     * <p>
     * &lt;jsp:getProperty name=<i>"id"</i> property="path"/&gt;
     *
     * @return String - cookie path on the server
     */
    public final String getPath()
    {
	return cookie.getPath();
    }

    /**
     * Returns the value of the cookie.
     * <p>
     * &lt;jsp:getProperty name=<i>"id"</i> property="value"/&gt;
     *
     * @return String - cookie value
     */
    public final String getValue()
    {
	return cookie.getValue();
    }

    /**
     * Returns "1" if the browser is sending cookies only over a secure
     * protocol, or "0" if the browser can send cookies using any protocol.
     * <p>
     * &lt;jsp:getProperty name=<i>"id"</i> property="secure"/&gt;
     *
     * @return String - "1" if the cookies are secure, "0" if not
     */
    public final String getSecure()
    {
	if( cookie.getSecure() )
	    return "1";
	return "0";
    }

    /**
     * Returns the version of the protocol this cookie complies with.
     * <p>
     * &lt;jsp:getProperty name=<i>"id"</i> property="version"/&gt;
     *
     * @return String - cookie version
     */
    public final String getVersion()
    {
	return "" + cookie.getVersion();
    }

}
