/*
 * $Header: /home/cvs/jakarta-taglibs/request/src/org/apache/taglibs/request/EqualsCookieTag.java,v 1.1 2001/07/13 00:08:59 glenn Exp $
 * $Revision: 1.1 $
 * $Date: 2001/07/13 00:08:59 $
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 */

package org.apache.taglibs.request;

import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;
import javax.servlet.jsp.*;
import javax.servlet.jsp.tagext.*;

/**
 * JSP Tag <b>equalsCookie</b>, used to determine if a Request
 * cookie equals the value of the "match" tag attribute.
 * <p>
 * Includes the body of the tag if the cookie equals the value of the 
 * "match" tag cookie.
 * <p>
 * You can set the optional tag cookie <b>value</b> to <i>true</i> or
 * <i>false</i>.  The body of the tag is included if equalsCookie matches
 * the value.
 * <p>
 * You can set the optional tag cookie <b>ignoreCase</b> to <i>true</i> or
 * <i>false</i>.  If ignoreCase is set to true, then the comparison between the
 * request cookie and the "match" tag attribute will <i>not</i> be 
 * case-sensitive.
 * <p>
 * JSP Tag Lib Descriptor
 * <p><pre>
 * &lt;name>equalsCookie&lt;/name>
 * &lt;tagclass>org.apache.taglibs.request.EqualsCookieTag&lt;/tagclass>
 * &lt;bodycontent>JSP&lt;/bodycontent>
 * &lt;info>Includes the body of the tag if the request cookie equals the
 * value of the "match" tag cookie.&lt;/info>
 *   &lt;attribute>
 *     &lt;name>name&lt;/name>
 *     &lt;required>true&lt;/required>
 *     &lt;rtexprvalue>false&lt;/rtexprvalue>
 *   &lt;/attribute>
 *   &lt;attribute>
 *     &lt;name>value&lt;/name>
 *     &lt;required>false&lt;/required>
 *     &lt;rtexprvalue>false&lt;/rtexprvalue>
 *   &lt;/attribute>
 *   &lt;attribute>
 *     &lt;name>match&lt;/name>
 *     &lt;required>true&lt;/required>
 *     &lt;rtexprvalue>true&lt;/rtexprvalue>
 *   &lt;/attribute>
 *   &lt;attribute>
 *     &lt;name>ignoreCase&lt;/name>
 *     &lt;required>false&lt;/required>
 *     &lt;rtexprvalue>false&lt;/rtexprvalue>
 *   &lt;/attribute>
 * </pre>
 *
 * @author Glenn Nielsen
 */


public class EqualsCookieTag extends TagSupport
{
    private String name = null;
    private String match = null;
    //default behaviour is don't ignore case and execute body when tag contents
    //equal value set by setMatch
    private boolean ignoreCase = false;
    private boolean value = true;

    /**
     * Includes the body of the tag if the request cookie equals the value set in the 
     * 'match' cookie.
     *
     * @return SKIP_BODY if equalsCookie body content does not equal the value of 
     * the match cookie, EVAL_BODY_include if it does
     */
    public final int doStartTag() throws JspException
    {
        //result is whether or not tag contents equal the match cookie
        boolean result = false;
        // Get the cookies
        Cookie cookie = null;
        Cookie [] cookies =
            ((HttpServletRequest)pageContext.getRequest()).getCookies();
        if( cookies != null ) {
            for( int i = 0; i < cookies.length; i++ ) {
                if( cookies[i].getName().equals(name) ) {
                    cookie = cookies[i];
                    break;
                }
            }    
        }    
        
        if (cookie != null) {
            if (ignoreCase) {
                result = cookie.getValue().equalsIgnoreCase(match);
            } else {
                result = cookie.getValue().equals(match);
            }
        }
        
        if( value == result )
            return EVAL_BODY_INCLUDE;

        return SKIP_BODY;
    }

    /**
     * Set the required tag cookie <b>name</b>. 
     *
     * @param String name of request cookie
     */
    public final void setName(String str)
    {   
	name = str;
    }


    /**
     * Set the String that will be compared to the request
     * cookie.
     *
     * @param String value to match against the request cookie
     */
    public final void setMatch(String str) {
        match=str;
    }
    

    /**
     * If ignoreCase is set to true, then the comparison 
     * between the "match" attribute and the 
     * request cookie will <i>not</i> be case sensitive
     * 
     * @param boolean    true = ignore case<BR>false = case sensitive<BR>
     *               default value = false
     */
    public final void setIgnoreCase(boolean value) {
        ignoreCase = value;
    }
    
    /**
     * Set the optional tag cookie <b>value</b> to true or false. 
     *
     * @param boolean true or false
     */
    public final void setValue(boolean value)
    {
	this.value = value;
    }

}
