/*
 * $Header: /home/cvs/jakarta-taglibs/request/src/org/apache/taglibs/request/HeadersTag.java,v 1.3 2001/07/13 00:09:03 glenn Exp $
 * $Revision: 1.3 $
 * $Date: 2001/07/13 00:09:03 $
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 */

package org.apache.taglibs.request;

import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;
import javax.servlet.jsp.*;
import javax.servlet.jsp.tagext.*;

/**
 * JSP Tag <b>headers</b>, used to get HttpServletRequest header information
 * using the standard JSP &lt;jsp:getProperty&gt; tag.
 * <p>
 * The script variable of name <b>id</b> is availble only within the
 * body of the <b>headers</b> tag.
 * <p>
 * Loops through all the headers received by the HttpServletRequest.
 * <p>
 * If the optional attribute <b>name</b> is present only the header of
 * that name is retreived.
 * <p>
 * JSP Tag Lib Descriptor
 * <p><pre>
 * &lt;name&gt;headers&lt;/name&gt;
 * &lt;tagclass&gt;org.apache.taglibs.request.HeadersTag&lt;/tagclass&gt;
 * &lt;teiclass&gt;org.apache.taglibs.request.HeadersTEI&lt;/teiclass&gt;
 * &lt;bodycontent&gt;JSP&lt;/bodycontent&gt;
 * &lt;info&gt;Loop through all headers, or get the properties of a single header.&lt;/info&gt;
 *   &lt;attribute&gt;
 *     &lt;name&gt;id&lt;/name&gt;
 *     &lt;required&gt;true&lt;/required&gt;
 *     &lt;rtexprvalue&gt;false&lt;/rtexprvalue&gt;
 *   &lt;/attribute&gt;
 *   &lt;attribute&gt;
 *     &lt;name&gt;name&lt;/name&gt;
 *     &lt;required&gt;false&lt;/required&gt;
 *     &lt;rtexprvalue&gt;false&lt;/rtexprvalue&gt;
 *   &lt;/attribute&gt;
 * </pre>
 *
 * @author Glenn Nielsen
 */

public class HeadersTag extends BodyTagSupport
{
    // Optional attribute, name of HTTP header to get
    private String name = null;
    // The current request
    private HttpServletRequest req = null;
    // All the headers for request
    private Enumeration headers = null;
    // Name of current header
    private String header = null;

    /**
     * Gets the headers that came with the request or header with <b>name</b>.
     *
     * @return SKIP_BODY if no headers or header with <b>name</b> is not found, EVAL_BODY_TAG if header exists
     */
    public final int doStartTag() throws JspException
    {
	// Get the request
	req = (HttpServletRequest)pageContext.getRequest();

	if( name != null ) {
	    if( req.getHeader(name) != null )
		header = name;
	} else {
	    headers = req.getHeaderNames();
	    if( headers == null || !headers.hasMoreElements() )
		return SKIP_BODY;
	    header = (String)headers.nextElement();
	}
	if( header == null )
	    return SKIP_BODY;

	pageContext.setAttribute(id,this,PageContext.PAGE_SCOPE);
	return EVAL_BODY_TAG;
    }

    /**
     * Method called at end of each headers tag body.
     *
     * @return EVAL_BODY_TAG if there is another header, or SKIP_BODY if there are no more headers or this is a named header
     */
    public final int doAfterBody() throws JspException
    {
	// See if this is the last or a named header
	if( name != null || !headers.hasMoreElements() )
	    return SKIP_BODY;
	// There is another header, so loop again
	header = (String)headers.nextElement();
	if( header == null )
	    return SKIP_BODY;
	return EVAL_BODY_TAG;
    }

    /**
     * Method called at end of Tag
     *
     * @return EVAL_PAGE
     */
    public final int doEndTag() throws JspException
    {
        pageContext.removeAttribute(id,PageContext.PAGE_SCOPE);
	try
	{
	    if(bodyContent != null)
	    bodyContent.writeOut(bodyContent.getEnclosingWriter());
	} catch(java.io.IOException e)
	{
	    throw new JspException("IO Error: " + e.getMessage());
	}
	return EVAL_PAGE;
    }

    /**
     * Set the optional tag attribute <b>name</b>.
     *
     * @param String name of HTTP header
     */
    public final void setName(String str)
    {
	name = str;
    }

    /**
     * Returns the name of the header.
     * <p>
     * &lt;jsp:getProperty name=<i>"id"</i> property="name"/&gt;
     *
     * @return String - header name
     */
    public final String getName()
    {
	return header;
    }

    /**
     * Returns the value of the header.
     * <p>
     * &lt;jsp:getProperty name=<i>"id"</i> property="header"/&gt;
     *
     * @return String - value of the header
     */
    public final String getHeader()
    {
	String value = req.getHeader(header);
	if( value == null )
	    return "";
	return value;
    }

    /**
     * Returns the value of a date header as the number of ms since the epoch.
     * <p>
     * &lt;jsp:getProperty name=<i>"id"</i> property="dateHeader"/&gt;
     *
     * @return String - value of a date header
     */
    public final String getDateHeader()
    {
	return "" + req.getDateHeader(header);
    }

    /**
     * Returns the value of a header as an integer.
     * <p>
     * &lt;jsp:getProperty name=<i>"id"</i> property="intHeader"/&gt;
     *
     * @return String - value of an integer header
     */
    public final String getIntHeader()
    {
	return "" + req.getIntHeader(header);
    }

}
