/*
 * $Header: /home/cvs/jakarta-taglibs/request/src/org/apache/taglibs/request/ParameterValuesTag.java,v 1.4 2002/04/04 19:07:53 glenn Exp $
 * $Revision: 1.4 $
 * $Date: 2002/04/04 19:07:53 $
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 */

package org.apache.taglibs.request;

import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;
import javax.servlet.jsp.*;
import javax.servlet.jsp.tagext.*;

/**
 * JSP Tag <b>parameterValues</b>, used to get HTTP parameter values for
 * parameters which have multiple values
 * using the standard JSP &lt;jsp:getProperty&gt; tag.
 * <p>
 * The script variable of name <b>id</b> is availble only within the
 * body of the <b>parameterValues</b> tag.
 * <p>
 * Loops through all the parameter values for a parameter.
 * <p>
 * Must be nested within a <b>parameters</b> tag.
 * <p>
 * JSP Tag Lib Descriptor
 * <p><pre>
 * &lt;name&gt;parameterValues&lt;/name&gt;
 * &lt;tagclass&gt;org.apache.taglibs.request.ParameterValuesTag&lt;/tagclass&gt;
 * &lt;teiclass&gt;org.apache.taglibs.request.ParameterValuesTEI&lt;/teiclass&gt;
 * &lt;bodycontent&gt;JSP&lt;/bodycontent&gt;
 * &lt;info&gt;Used to get the name and values of a single parameter.&lt;/info&gt;
 *   &lt;attribute&gt;
 *     &lt;name&gt;id&lt;/name&gt;
 *     &lt;required&gt;true&lt;/required&gt;
 *     &lt;rtexprvalue&gt;false&lt;/rtexprvalue&gt;
 *   &lt;/attribute&gt;
 * </pre>
 *
 * @author Glenn Nielsen
 */

public class ParameterValuesTag extends BodyTagSupport
{
    private String [] values = null;
    private String value = null;
    private int parameter_num = 0;

    /**
     * Gets the parameter values for the current parameter.
     *
     * @return SKIP_BODY if no parameter values exist, EVAL_BODY_TAG if parameter values exist
     */
    public final int doStartTag() throws JspException
    {
        // Initialize variables
        values = null;
        value = null;
        parameter_num = 0;

	// Get the parent parameters tag
	ParametersTag pt;
	try {
	    pt = (ParametersTag)this.findAncestorWithClass(this,
		Class.forName("org.apache.taglibs.request.ParametersTag"));
	} catch(Exception e) {
	    return SKIP_BODY;
	}

	// Get the parameters
	values = ((HttpServletRequest)pageContext.getRequest()).getParameterValues(pt.getName());
	if( values == null || values.length == 0 )
	    return SKIP_BODY;

	value = values[0];
	if( value == null )
	    return SKIP_BODY;

	pageContext.setAttribute(id,this,PageContext.PAGE_SCOPE);
	return EVAL_BODY_TAG;
    }

    /**
     * Method called at end of each parameterValues tag.
     *
     * @return EVAL_BODY_TAG if there is another parameter value, or SKIP_BODY if there are no more parameter values
     */
    public final int doAfterBody() throws JspException
    {
	parameter_num++;
	// See if this is the last parameter value
	if( parameter_num >= values.length )
	    return SKIP_BODY;
	// There is another parameter value, so loop again
	value = values[parameter_num];
	if( value == null )
	    return SKIP_BODY;
	return EVAL_BODY_TAG;
    }

    /**
     * Method called at end of Tag
     * @return EVAL_PAGE
     */
    public final int doEndTag() throws JspException
    {
        pageContext.removeAttribute(id,PageContext.PAGE_SCOPE);
	try
	{
	    if(bodyContent != null)
	    bodyContent.writeOut(bodyContent.getEnclosingWriter());
	} catch(java.io.IOException e)
	{
	    throw new JspException("IO Error: " + e.getMessage());
	}
	return EVAL_PAGE;
    }

    /**
     * Returns the value of the parameter.
     * <p>
     * &lt;jsp:getProperty name=<i>"id"</i> property="value"/&gt;
     *
     * @return String - value of the parameter
     */
    public final String getValue()
    {
	return value;
    }

}
