/*
 * $Header: /home/cvs/jakarta-taglibs/request/src/org/apache/taglibs/request/ParametersTag.java,v 1.3 2001/07/13 00:09:06 glenn Exp $
 * $Revision: 1.3 $
 * $Date: 2001/07/13 00:09:06 $
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 */

package org.apache.taglibs.request;

import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;
import javax.servlet.jsp.*;
import javax.servlet.jsp.tagext.*;

/**
 * JSP Tag <b>parameters</b>, used to get HttpServletRequest parameter information
 * using the standard JSP &lt;jsp:getProperty&gt; tag.
 * <p>
 * The script variable of name <b>id</b> is availble only within the
 * body of the <b>parameters</b> tag.
 * <p>
 * Loops through all the parameters received by the HttpServletRequest.
 * <p>
 * If the optional attribute <b>name</b> is present only the parameter of
 * that name is retreived.
 * <p>
 * JSP Tag Lib Descriptor
 * <p><pre>
 * &lt;name&gt;parameters&lt;/name&gt;
 * &lt;tagclass&gt;org.apache.taglibs.request.ParametersTag&lt;/tagclass&gt;
 * &lt;teiclass&gt;org.apache.taglibs.request.ParametersTEI&lt;/teiclass&gt;
 * &lt;bodycontent&gt;JSP&lt;/bodycontent&gt;
 * &lt;info&gt;Used to loop through all parameters or get a single parameter.&lt;/info&gt;
 *   &lt;attribute&gt;
 *     &lt;name&gt;id&lt;/name&gt;
 *     &lt;required&gt;true&lt;/required&gt;
 *     &lt;rtexprvalue&gt;false&lt;/rtexprvalue&gt;
 *   &lt;/attribute&gt;
 *   &lt;attribute&gt;
 *     &lt;name&gt;name&lt;/name&gt;
 *     &lt;required&gt;false&lt;/required&gt;
 *     &lt;rtexprvalue&gt;false&lt;/rtexprvalue&gt;
 *   &lt;/attribute&gt;
 * </pre>
 *
 * @author Glenn Nielsen
 */

public class ParametersTag extends BodyTagSupport
{
    // Optional attribute, HTTP input parameter name
    private String name = null;
    private HttpServletRequest req = null;
    // All the parameter names
    private Enumeration parameters = null;
    // Name of current parameter
    private String parameter = null;

    /**
     * Gets the parameters that came with the request or parameter with <b>name</b>.
     *
     * @return SKIP_BODY if no parameters or parameter with <b>name</b> is not found, EVAL_BODY_TAG if parameter exists
     */
    public final int doStartTag() throws JspException
    {
	// Get the request
	req = (HttpServletRequest)pageContext.getRequest();

	if( name != null ) {
	    if( req.getParameter(name) != null )
		parameter = name;
	} else {
	    parameters = req.getParameterNames();
	    if( parameters == null || !parameters.hasMoreElements() )
		return SKIP_BODY;
	    parameter = (String)parameters.nextElement();
	}
	if( parameter == null )
	    return SKIP_BODY;

	pageContext.setAttribute(id,this,PageContext.PAGE_SCOPE);
	return EVAL_BODY_TAG;
    }

    /**
     * Method called at end of each parameters tag.
     *
     * @return EVAL_BODY_TAG if there is another parameter, or SKIP_BODY if there are no more parameters or this is a named parameter
     */
    public final int doAfterBody() throws JspException
    {
	// See if this is the last or a named parameter
	if( name != null || !parameters.hasMoreElements() )
	    return SKIP_BODY;
	// There is another parameter, so loop again
	parameter = (String)parameters.nextElement();
	if( parameter == null )
	    return SKIP_BODY;
	return EVAL_BODY_TAG;
    }

    /**
     * Method called at end of Tag
     * @return EVAL_PAGE
     */
    public final int doEndTag() throws JspException
    {
        pageContext.removeAttribute(id,PageContext.PAGE_SCOPE);
	try
	{
	    if(bodyContent != null)
	    bodyContent.writeOut(bodyContent.getEnclosingWriter());
	} catch(java.io.IOException e)
	{
	    throw new JspException("IO Error: " + e.getMessage());
	}
	return EVAL_PAGE;
    }

    /**
     * Set the optional tag attribute <b>name</b>.
     *
     * @param String name of HTTP input parameter
     */
    public final void setName(String str)
    {
	name = str;
    }

    /**
     * Returns the name of the parameter.
     * <p>
     * &lt;jsp:getProperty name=<i>"id"</i> property="name"/&gt;
     *
     * @return String - parameter name
     */
    public final String getName()
    {
	return parameter;
    }

    /**
     * Returns the value of the parameter.
     * <p>
     * &lt;jsp:getProperty name=<i>"id"</i> property="value"/&gt;
     *
     * @return String - value of the parameter
     */
    public final String getValue()
    {
	String value = req.getParameter(parameter);
	if( value == null )
	    return "";
	return value;
    }

}
