/*
 * $Header: /home/cvs/jakarta-taglibs/request/src/org/apache/taglibs/request/RequestData.java,v 1.2 2000/12/04 01:22:09 glenn Exp $
 * $Revision: 1.2 $
 * $Date: 2000/12/04 01:22:09 $
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 */

package org.apache.taglibs.request;

import javax.servlet.*;
import javax.servlet.http.*;
import javax.servlet.jsp.*;
import javax.servlet.jsp.tagext.*;

/**
 * Stores the scripting variable data provided by the JSP Tag
 * <b>request</b>, used to access HttpServletRequest information using
 * the standard JSP &lt;jsp:getProperty&gt; tag.
 * <p>
 * The script variable of name <b>id</b> is available for use during
 * processing of the remainder of the JSP page.
 * <p>
 *
 * @see RequestTag
 *
 * @author Glenn Nielsen
 */

public class RequestData
{
    private HttpServletRequest req;

    RequestData(HttpServletRequest request)
    {
        this.req = request;
    }

    /**
     * Returns the name of the authentication scheme used to protect the
     * servlet.
     * <p>
     * &lt;jsp:getProperty name=<i>"id"</i> property="authType"/&gt;
     *
     * @return String - "BASIC", "SSL," or "" if the servlet was not protected.
     */
    public final String getAuthType()
    {
	return req.getAuthType();
    }

    /**
     * Returns the portion of the request URI that indicates the context
     * of the request.
     * <p>
     * &lt;jsp:getProperty name=<i>"id"</i> property="contextPath"/&gt;
     *
     * @return String - request URI context path
     */
    public final String getContextPath()
    {
	return req.getContextPath();
    }

    /**
     * Returns the name of the HTTP method with which this request was made.
     * <p>
     * &lt;jsp:getProperty name=<i>"id"</i> property="method"/&gt;
     *
     * @return String - "GET", "POST", or "PUT".
     */
    public final String getMethod()
    {
	return req.getMethod();
    }

    /**
     * Returns any extra path information associated with the URL the
     * client sent when it made this request.
     * <p>
     * &lt;jsp:getProperty name=<i>"id"</i> property="pathInfo"/&gt;
     *
     * @return String - path info portion of the request URL
     */
    public final String getPathInfo()
    {
	return req.getPathInfo();
    }

    /**
     * Returns any extra path information after the servlet name but
     * before the query string, and translates it to a real path.
     * <p>
     * &lt;jsp:getProperty name=<i>"id"</i> property="pathTranslated"/&gt;
     *
     * @return String - translated path information
     */
    public final String getPathTranslated()
    {
	return req.getPathTranslated();
    }

    /**
     * Returns the query string that is contained in the request URL
     * after the path.
     * <p>
     * &lt;jsp:getProperty name=<i>"id"</i> property="queryString"/&gt;
     *
     * @return String - query string from the URL
     */
    public final String getQueryString()
    {
	return req.getQueryString();
    }

    /**
     * Returns the login of the user making this request, if the user has
     * been authenticated, or "" if the user has not been authenticated.
     * <p>
     * &lt;jsp:getProperty name=<i>"id"</i> property="remoteUser"/&gt;
     *
     * @return String - remote user name or "" String
     */
    public final String getRemoteUser()
    {
	return req.getRemoteUser();
    }

    /**
     * Returns the session ID specified by the client.
     * <p>
     * &lt;jsp:getProperty name=<i>"id"</i> property="requestedSessionId"/&gt;
     *
     * @return String - session ID
     */
    public final String getRequestedSessionId()
    {
	return req.getRequestedSessionId();
    }

    /**
     * Returns the part of this request's URL from the protocol name up
     * to the query string in the first line of the HTTP request.
     * <p>
     * &lt;jsp:getProperty name=<i>"id"</i> property="requestURI"/&gt;
     *
     * @return String - request URI
     */
    public final String getRequestURI()
    {
	return req.getRequestURI();
    }

    /** 
     * Returns the request URL up
     * to the query string in the first line of the HTTP request.
     * <p>
     * &lt;jsp:getProperty name=<i>"id"</i> property="requestURL"/&gt;  
     * 
     * @return String - request URL
     */
    public final String getRequestURL()
    {
        return "" + HttpUtils.getRequestURL(req);
    }

    /**
     * Returns the part of this request's URL that calls the servlet.
     * <p>
     * &lt;jsp:getProperty name=<i>"id"</i> property="servletPath"/&gt;
     *
     * @return String - servlet path
     */
    public final String getServletPath()
    {
	return req.getServletPath();
    }

    /**
     * Returns the name of the character encoding used in the body of
     * this request.
     * <p>
     * &lt;jsp:getProperty name=<i>"id"</i> property="characterEncoding"/&gt;
     *
     * @return String - character encoding
     */
    public final String getCharacterEncoding()
    {
	return req.getCharacterEncoding();
    }

    /**
     * Returns the length, in bytes, of the request body and made available
     * by the input stream, or -1 if the length is not known.
     * <p>
     * &lt;jsp:getProperty name=<i>"id"</i> property="contentLength"/&gt;
     *
     * @return String - content length or "-1"
     */
    public final String getContentLength()
    {
	return "" + req.getContentLength();
    }

    /**
     * Returns the MIME type of the body of the request, or null if
     * the type is not known.
     * <p>
     * &lt;jsp:getProperty name=<i>"id"</i> property="contentType"/&gt;
     *
     * @return String - content type
     */
    public final String getContentType()
    {
	return req.getContentType();
    }

    /**
     * Returns the name and version of the protocol the request uses in the
     * form protocol/majorVersion.minorVersion, for example, HTTP/1.1.
     * <p>
     * &lt;jsp:getProperty name=<i>"id"</i> property="protocol"/&gt;
     *
     * @return String - protocol of the request
     */
    public final String getProtocol()
    {
	return req.getProtocol();
    }

    /**
     * Returns the Internet Protocol (IP) address of the client that
     * sent the request.
     * <p>
     * &lt;jsp:getProperty name=<i>"id"</i> property="remoteAddr"/&gt;
     *
     * @return String - remote IP address
     */
    public final String getRemoteAddr()
    {
	return req.getRemoteAddr();
    }

    /**
     * Returns the fully qualified name of the client that sent the request,
     * or the IP address of the client if the name cannot be determined.
     * <p>
     * &lt;jsp:getProperty name=<i>"id"</i> property="remoteHost"/&gt;
     *
     * @return String - remote host
     */
    public final String getRemoteHost()
    {
	return req.getRemoteHost();
    }

    /**
     * Returns the name of the scheme used to make this request, for
     * example, http, https, or ftp.
     * <p>
     * &lt;jsp:getProperty name=<i>"id"</i> property="scheme"/&gt;
     *
     * @return String - scheme used to make the request
     */
    public final String getScheme()
    {
	return req.getScheme();
    }

    /**
     * Returns the host name of the server that received the request.
     * <p>
     * &lt;jsp:getProperty name=<i>"id"</i> property="serverName"/&gt;
     *
     * @return String - host name of the server
     */
    public final String getServerName()
    {
	return req.getServerName();
    }

    /**
     * Returns the port number on which this request was received.
     * <p>
     * &lt;jsp:getProperty name=<i>"id"</i> property="serverPort"/&gt;
     *
     * @return String - server port number
     */
    public final String getServerPort()
    {
	return "" + req.getServerPort();
    }

}
