/*
 * $Header: /home/cvs/jakarta-taglibs/response/src/org/apache/taglibs/response/AddCookieTag.java,v 1.3 2001/07/15 16:21:22 glenn Exp $
 * $Revision: 1.3 $
 * $Date: 2001/07/15 16:21:22 $
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 */

package org.apache.taglibs.response;

import javax.servlet.*;
import javax.servlet.http.*;
import javax.servlet.jsp.*;
import javax.servlet.jsp.tagext.*;

/**
 * JSP Tag <b>addCookie</b>, used to add a cookie to the Http Response.
 * <p>
 * The script variable for the <b>name</b> of the cookie is required.
 * <p>
 * Cookie attributes such as the comment, value, etc. can be set
 * statically by using the corresponding addCookie tag attribute.
 * Or dynamically using the corresponding tag in the body of the
 * addCookie tag.
 * <p>
 * JSP Tag Lib Descriptor
 * <p><pre>
 * &lt;name&gt;addCookie&lt;/name&gt;
 * &lt;tagclass&gt;org.apache.taglibs.response.AddCookieTag&lt;/tagclass&gt;
 * &lt;bodycontent&gt;JSP&lt;/bodycontent&gt;
 * &lt;info&gt;Add a cookie to an Http Response.&lt;/info&gt;
 *   &lt;attribute&gt;
 *     &lt;name&gt;name&lt;/name&gt;
 *     &lt;required&gt;true&lt;/required&gt;
 *     &lt;rtexprvalue&gt;false&lt;/rtexprvalue&gt;
 *   &lt;/attribute&gt;
 *   &lt;attribute&gt;
 *     &lt;name&gt;value&lt;/name&gt;
 *     &lt;required&gt;false&lt;/required&gt;
 *     &lt;rtexprvalue&gt;false&lt;/rtexprvalue&gt;
 *   &lt;/attribute&gt;
 *   &lt;attribute&gt;
 *     &lt;name&gt;comment&lt;/name&gt;    
 *     &lt;required&gt;false&lt;/required&gt;
 *     &lt;rtexprvalue&gt;false&lt;/rtexprvalue&gt;
 *   &lt;/attribute&gt;
 *   &lt;attribute&gt;
 *     &lt;name&gt;domain&lt;/name&gt;    
 *     &lt;required&gt;false&lt;/required&gt;
 *     &lt;rtexprvalue&gt;false&lt;/rtexprvalue&gt;
 *   &lt;/attribute&gt;
 *   &lt;attribute&gt;
 *     &lt;name&gt;maxAge&lt;/name&gt;    
 *     &lt;required&gt;false&lt;/required&gt;
 *     &lt;rtexprvalue&gt;false&lt;/rtexprvalue&gt;
 *   &lt;/attribute&gt;
 *   &lt;attribute&gt;
 *     &lt;name&gt;path&lt;/name&gt;    
 *     &lt;required&gt;false&lt;/required&gt;
 *     &lt;rtexprvalue&gt;false&lt;/rtexprvalue&gt;
 *   &lt;/attribute&gt;
 *   &lt;attribute&gt;
 *     &lt;name&gt;secure&lt;/name&gt;    
 *     &lt;required&gt;false&lt;/required&gt;
 *     &lt;rtexprvalue&gt;false&lt;/rtexprvalue&gt;
 *   &lt;/attribute&gt;
 *   &lt;attribute&gt;
 *     &lt;name&gt;version&lt;/name&gt;    
 *     &lt;required&gt;false&lt;/required&gt;
 *     &lt;rtexprvalue&gt;false&lt;/rtexprvalue&gt;
 *   &lt;/attribute&gt;
 * </pre>
 *
 * @author Glenn Nielsen
 */

public class AddCookieTag extends TagSupport
{
    private Cookie cookie = null;
    private String name = null;
    private String value = null;
    private String comment = null;
    private String domain = null;
    private int maxAge = -1;
    private String path = null;
    private boolean secure = false;
    private int version = 0;

    /**
     * Method called at start of addCookie tag to create cookie
     *
     * @return EVAL_BODY_INCLUDE
     */
    public final int doStartTag() throws JspException
    {
	cookie = new Cookie(name,"");
	return EVAL_BODY_INCLUDE;
    }

    /**
     * Method called at end of Tag used to set cookie attributes
     * and add the cookie to the Http Response.
     *
     * @return EVAL_PAGE
     */
    public final int doEndTag() throws JspException
    {
	// Set the cookie attribute values
	if( value != null )
	    cookie.setValue(value);
	if( comment != null )
	    cookie.setComment(comment);
	if( domain != null )
	    cookie.setDomain(domain);
	cookie.setMaxAge(maxAge);
	cookie.setSecure(secure);
	cookie.setVersion(version);
	// Add the cookie
	((HttpServletResponse)pageContext.getResponse()).addCookie(cookie);

	return EVAL_PAGE;
    }

    /**
     * Optional attribute, specifies a comment that describes a cookie's purpose.
     *
     * @param String comment
     */
    public final void setComment(String com)
    {
	comment = com;
    }

    /**
     * Optional attribute, specifies the domain within which this
     * cookie should be presented.
     *
     * @param String domain
     */
    public final void setDomain(String dom)
    {
	domain = dom;
    }

    /**
     * Optional attribute, sets the maximum age of the cookie in seconds.
     *
     * @param String number of seconds
     */
    public final void setMaxAge(int max)
    {
        maxAge = max;
    }

    /**
     * Required attribute, name of the cookie.
     *
     * @param String name
     */
    public final void setName(String nam)
    {
	name = nam;
    }

    /**
     * Optional attribute, specifies a path for the cookie to which
     * the client should return the cookie.
     *
     * @param String path
     */
    public final void setPath(String pth)
    {
	path = pth;
    }

    /**
     * Optional attribute, assigns a value to a cookie.
     *
     * @param String value
     */
    public final void setValue(String val)
    {
	value = val;
    }

    /**
     * Optional attribute, indicates to the browser whether the cookie
     * should only be sent using a secure protocol, such as HTTPS or SSL.
     * Set to true or false.  Default value is false.
     *
     * @param String true or false
     */
    public final void setSecure(boolean flag)
    {
        secure = flag;
    }

    /**
     * Optional attribute, sets the version of the cookie protocol
     * this cookie complies with.
     *
     * @param int version number
     */
    public final void setVersion(int version)
    {
        this.version = version;
    }

}
