/*
 * $Header: /home/cvs/jakarta-taglibs/response/src/org/apache/taglibs/response/SendErrorTag.java,v 1.3 2001/07/15 16:21:24 glenn Exp $
 * $Revision: 1.3 $
 * $Date: 2001/07/15 16:21:24 $
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 */

package org.apache.taglibs.response;

import java.io.*;
import java.lang.*;
import java.lang.reflect.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;
import javax.servlet.jsp.*;
import javax.servlet.jsp.tagext.*;

/**
 * JSP Tag <b>sendError</b>, used to send error as the HTTP Response.
 * <p>
 * The <b>sendError</b> tag can use an optional error message that it
 * gets from the body of the tag.
 * <p>
 * The required attribute <b>error</b> must be set to an HTTP error
 * code such as <b>SC_UNAUTHORIZED</b>.
 * <p>
 * If the optional attribute <b>reset</b> is set to <b>true</b> the
 * buffer, headers, and status code will be reset.
 * <p>
 * JSP Tag Lib Descriptor
 * <p><pre>
 * &lt;name&gt;sendError&lt;/name&gt;
 * &lt;tagclass&gt;org.apache.taglibs.response.SendErrorTag&lt;/tagclass&gt;
 * &lt;bodycontent&gt;JSP&lt;/bodycontent&gt;
 * &lt;info&gt;Return an HTTP error as the HTTP Response.&lt;/info&gt;
 *   &lt;attribute&gt;
 *     &lt;name&gt;error&lt;/name&gt;
 *     &lt;required&gt;true&lt;/required&gt;
 *     &lt;rtexprvalue&gt;false&lt;/rtexprvalue&gt;
 *   &lt;/attribute&gt;
 *   &lt;attribute&gt;
 *     &lt;name&gt;reset&lt;/name&gt;
 *     &lt;required&gt;false&lt;/required&gt;
 *     &lt;rtexprvalue&gt;false&lt;/rtexprvalue&gt;
 *   &lt;/attribute&gt;
 * </pre>
 *
 * @author Glenn Nielsen
 */

public class SendErrorTag extends BodyTagSupport
{
    private String error;
    private boolean reset = false;

    /**
     * Method called at start of tag, just returns EVAL_BODY_TAG
     *
     * @return EVAL_BODY_TAG
     */
    public final int doStartTag() throws JspException
    {
	return EVAL_BODY_TAG;
    }

    /**
     * Read the body of the sendError tag to obtain the error message.
     *
     * @return SKIP_BODY
     */
    public final int doAfterBody() throws JspException
    {
        // Use the body of the tag as header value
        int error_code = 0;
        BodyContent body = getBodyContent();
        String s = body.getString().trim();
        // Clear the body since we only used it as input for the header
        // value
        body.clearBody();

	if( reset )
	    pageContext.getResponse().reset();

	try {
            HttpServletResponse resp = null;
            Field ec = null;
            resp = (HttpServletResponse)pageContext.getResponse();
            Class rc = resp.getClass();
            ec = rc.getField(error);
            error_code = ec.getInt(resp);
        } catch(Exception e) {
            throw new JspException(
                "Response sendError tag could not find error code: " + error);
        }

        try {
	    if( s.length() > 0 )
	        ((HttpServletResponse)pageContext.getResponse()).sendError(error_code,s);
	    else
		((HttpServletResponse)pageContext.getResponse()).sendError(error_code);
	} catch(IOException e) {
	    throw new JspException(
		"Response sendError tag failed: " + e.getMessage());
	} catch(IllegalStateException e) {
	    throw new JspException(
		"Response sendError tag failed " + e.getMessage());
	}

        return SKIP_BODY;
    }

    /**
     * Method called at end of Tag used to send an error code,
     * always returns SKIP_PAGE.
     *
     * @return SKIP_PAGE
     */ 
    public final int doEndTag() throws JspException
    {
	return SKIP_PAGE;
    }

    /**
     * Required attribute that sets the error code to return.
     *
     * The error must be a text string for an error code as
     * defined in java class HttpServletResponse. For example,
     * <b>SC_UNATHORIZED</b> to return HTTP error code 401.
     *
     * @param String error code, i.e. <b>SC_UNATHORIZED</b>
     */
    public final void setError(String err)
    {
        error = err;
    }

    /**
     * Optional attribute that flags whether the buffer, headers,
     * and status code should be reset.
     *
     * @param boolean <b>true</b> or <b>false</b>
     */
    public final void setReset(boolean reset)
    {
        this.reset = reset;
    }
}
