/*
 * $Header: /home/cvs/jakarta-taglibs/response/src/org/apache/taglibs/response/SetStatusTag.java,v 1.3 2001/07/15 16:21:25 glenn Exp $
 * $Revision: 1.3 $
 * $Date: 2001/07/15 16:21:25 $
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 */

package org.apache.taglibs.response;

import java.io.*;
import java.lang.*;
import java.lang.reflect.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;
import javax.servlet.jsp.*;
import javax.servlet.jsp.tagext.*;

/**
 * JSP Tag <b>setStatus</b>, used to set the status code for the HTTP Response.
 * <p>
 * JSP Tag Lib Descriptor
 * <p><pre>
 * &lt;name&gt;setStatus&lt;/name&gt;
 * &lt;tagclass&gt;org.apache.taglibs.response.SetStatusTag&lt;/tagclass&gt;
 * &lt;bodycontent&gt;empty&lt;/bodycontent&gt;
 * &lt;info&gt;Set the status code for the HTTP Response.&lt;/info&gt;
 *   &lt;attribute&gt;
 *     &lt;name&gt;status&lt;/name&gt;
 *     &lt;required&gt;true&lt;/required&gt;
 *     &lt;rtexprvalue&gt;false&lt;/rtexprvalue&gt;
 *   &lt;/attribute&gt;
 * </pre>
 *
 * @author Glenn Nielsen
 */

public class SetStatusTag extends TagSupport
{
    private String status;

    /**
     * Method called at end of tag which sets the HTTP Response status code.
     *
     * @return SKIP_BODY
     */
    public final int doEndTag() throws JspException
    {
        int status_code = 0;
        HttpServletResponse resp = null;
        Field ec = null;
 
        try {
            resp = (HttpServletResponse)pageContext.getResponse(); 
            Class rc = resp.getClass();
            ec = rc.getField(status);
            status_code = ec.getInt(resp);
        } catch(Exception e) {
            throw new JspException(
                "Response setStatus tag could not find status code: " + status);
        }

	((HttpServletResponse)pageContext.getResponse()).setStatus(status_code);
        return EVAL_PAGE;
    }

    /**
     * Required attribute status, set the status code to return.
     *
     * The status must be a text string for a status code as
     * defined in java class HttpServletResponse. For example,
     * <b>SC_OK</b> to return HTTP status code 200.
     *
     * @param String Status Code name, e.g. "SC_OK"
     */
    public final void setStatus(String stat)
    {
        status = stat;
    }
}
