/*
 * $Header: /home/cvs/jakarta-taglibs/scrape/src/org/apache/taglibs/scrape/ScrapeData.java,v 1.1.1.1 2001/02/27 03:00:39 glenn Exp $
 * $Revision: 1.1.1.1 $
 * $Date: 2001/02/27 03:00:39 $
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 */

package org.apache.taglibs.scrape;

import java.util.*;

/**
 * ScrapeData - An object used to store data for a scrape, data stored is a
 *                scrapeid, begin and end markers, flags, and the results of
 *                the scrape 
 *
 * @author Rich Catlett
 *
 * @version 1.0
 *
 */
public class ScrapeData {

    /**
     * flag determines if tags (anything enclosed by < >) are kept in the result
     */
    private boolean stripflag = false;
    /**
     * flag determines if begin and end markers are kept in the result default
     * value of false says they are not kept
     */
    private boolean anchorsflag = false;
    /**
     * beginning marker of this request
     */
    private String begin = new String();
    /**
     * end marker of this request
     */
    private String end = new String();
    /**
     * the result of this particular scrape
     */
    private StringBuffer result = new StringBuffer("");

    /**
     * constructor simply creates an instance of ScrapeData
     *
     */
    public ScrapeData() {}

    /**
     * getter method for stripflag
     *
     * @return boolean value of the Boolean object stripflag
     *
     */
    public final boolean getstripFlag() {
	return stripflag;
    }

    /**
     * setter method for stripflag
     *
     * @param notags  a boolean value that determines if tags are kept in the
     *                result
     */
    public final void setstripFlag(String notags) {
	if (notags.equalsIgnoreCase("true"))
	    stripflag = true;
	else 
	    stripflag = false;
    }

    /**
     * getter method for anchorsflag
     *
     * @return boolean value of anchorsflag
     *
     */
    public final boolean getanchorsFlag() {
	return anchorsflag;
    }

    /**
     * setter method for anchorsflag
     *
     * @param markers  a boolean value that determines if begin and end tags
     *                 are kept in the result
     */
    public final void setanchorsFlag(String markers) {
	if (markers.equalsIgnoreCase("true"))
	    anchorsflag = true;
	else
	    anchorsflag = false;
    }

    /**
     * setter method for the beginning anchor for this scrape    
     *
     * @param begin  anchor for the beginning of this scrape
     *
     */
    public final void setBegin(String begin) {
      this.begin = begin;
    }

    /**
     * getter method for the beginning anchor for this scrape
     *
     * @return the begin anchor for this scrape
     *
     */
    public final String getBegin() {
	return begin;
    }

    /**
     * setter method for the ending anchor for this scrape
     *
     * @param end  anchor for the end of this scrape
     */
    public final void setEnd(String end) {
      this.end = end;
    }

    /**
     * getter method for the ending anchor for this scrape
     *
     * @return the end anchor for this scrpae
     *
     */
    public final String getEnd() {
	return end;
    }

    /**
     * setter method for result
     *
     * @param result  the result of the scrape with this begin and end
     *
     */
    public final void setResult(String result) {
	this.result.setLength(0); // make sure that result is empty
	this.result.append(result);
    }

    /**
     * getter method for result
     *
     * @return the result of the scrape with this begin and end
     *
     */
    public final String getResult() {
	return result.toString();
    }
}
