/*
 * $Header: /home/cvs/jakarta-taglibs/string/src/org/apache/taglibs/string/OverlayTag.java,v 1.6 2003/04/27 04:34:12 bayard Exp $
 * $Revision: 1.6 $
 * $Date: 2003/04/27 04:34:12 $
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 */
package org.apache.taglibs.string;

import javax.servlet.jsp.JspException;
import org.apache.commons.lang.StringUtils;
import org.apache.commons.lang.NumberUtils;

/**
 * Overlay a String on top of another String. The length 
 * of the String to overlay does not have to be the same as 
 * the size of substring it replaces.
 *
 * <dl>
 * <dt>with</dt><dd>
 *             String to overlay onto the body.
 *             Required.
 * </dd>
 * <dt>start</dt><dd>
 *             Start index.
 *             Required.
 * </dd>
 * <dt>end</dt><dd>
 *             Index to end before.
 *             Required.
 * </dd>
 * </dl>
 * 
 * @author bayard@generationjava.com
 */
public class OverlayTag extends StringTagSupport {

    private String start;
    private String with;
    private String end;

    public OverlayTag() {
        super();
    }

    /**
     * Get the with property
     *
     * @return String property
     */
    public String getWith() {
        return this.with;
    }

    /**
     * Set the with property
     *
     * @param with String property
     */
    public void setWith(String with) {
        this.with = with;
    }


    /**
     * Get the start property
     *
     * @return String property
     */
    public String getStart() {
        return this.start;
    }

    /**
     * Set the start property
     *
     * @param start String property
     */
    public void setStart(String start) {
        this.start = start;
    }


    /**
     * Get the end property
     *
     * @return String property
     */
    public String getEnd() {
        return this.end;
    }

    /**
     * Set the end property
     *
     * @param end String property
     */
    public void setEnd(String end) {
        this.end = end;
    }



    public String changeString(String text) throws JspException {
        return StringUtils.overlayString(text, with, NumberUtils.stringToInt(start), NumberUtils.stringToInt(end));
    }

    public void initAttributes() {

        this.with = null;

        this.start = "0";

        this.end = "0";

    }

}
