/*
 * $Header: /home/cvs/jakarta-taglibs/string/src/org/apache/taglibs/string/RandomStringTag.java,v 1.6 2003/04/27 04:34:13 bayard Exp $
 * $Revision: 1.6 $
 * $Date: 2003/04/27 04:34:13 $
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 */
package org.apache.taglibs.string;

import javax.servlet.jsp.JspException;
import org.apache.commons.lang.RandomStringUtils;
import org.apache.commons.lang.NumberUtils;

// TODO: Decide if set should be a CharSet.

/// Note: There is a lot of tied logic between this class and 
///       the RandomStringUtils random methods. Understand how all the 
///       overloading works in RandomStringUtils before tackling this 
///       class.

/**
 * Create a random string. It can create an ascii string, 
 * a numeric string, an alphanumeric string, and a unicode 
 * string. The size of the string may be specified, as can a 
 * start and end character. Lastly, a set of characters to use 
 * may be passed in.
 *
 * <dl>
 * <dt>count</dt><dd>
 *            Number of characters to generate.
 *            Required.
 * </dd>
 * <dt>start</dt><dd>
 *            Start of range of characters to generate from.
 *            Linked to type attribute.
 * </dd>
 * <dt>end</dt><dd>
 *            End of range of characters to generate from.
 *            Linked to type attribute.
 * </dd>
 * <dt>type</dt><dd>
 *            Type of random-string. One of: numeric | alphanumeric | alphabet | unicode.
 *            Default value is to unicode.
 * </dd>
 * </dl>
 * 
 * @author bayard@generationjava.com
 */
public class RandomStringTag extends StringTagSupport {

    static public String NUMERIC      = "numeric";
    static public String ALPHANUMERIC = "alphanumeric";
    static public String ALPHABET     = "alphabet";
    static public String UNICODE      = "unicode";

    private String count;
    private String start;
    private String end;
    private String type;

    public RandomStringTag() {
        super();
    }

    /**
     * Get the count property
     *
     * @return String property
     */
    public String getCount() {
        return this.count;
    }

    /**
     * Set the count property
     *
     * @param with String property
     */
    public void setCount(String count) {
        this.count = count;
    }


    /**
     * Get the start property
     *
     * @return String property
     */
    public String getStart() {
        return this.start;
    }

    /**
     * Set the start property
     *
     * @param start String property
     */
    public void setStart(String start) {
        this.start = start;
    }


    /**
     * Get the end property
     *
     * @return String property
     */
    public String getEnd() {
        return this.end;
    }

    /**
     * Set the end property
     *
     * @param end String property
     */
    public void setEnd(String end) {
        this.end = end;
    }


    /**
     * Get the type property
     *
     * @return String property
     */
    public String getType() {
        return this.type;
    }

    /**
     * Set the type property
     *
     * @param with String property
     */
    public void setType(String type) {
        this.type = type;
    }

    public String changeString(String text) throws JspException {
        int st = 0;
        int ed = 0;
        int ct = 0;
        boolean letters = false;
        boolean numbers = false;

        letters = ALPHABET.equals(type);
        numbers = NUMERIC.equals(type);
        if(ALPHANUMERIC.equals(type)) {
            letters = true;
            numbers = true;
        }

        char[] setChrs = null;
        if( (text != null) && !text.equals("") ) {
            setChrs = text.toCharArray();
        }
        
        try {
            if(NumberUtils.isNumber(start)) {
                st = NumberUtils.createNumber(start).intValue();
            } else {
                st = (int)start.charAt(0);
            }
        } catch(NumberFormatException nfe) {
        } catch(NullPointerException npe) {   // easiest way to code it
        }
        try {
            if(NumberUtils.isNumber(end)) {
                ed = NumberUtils.createNumber(end).intValue();
            } else {
                ed = (int)end.charAt(0);
            }
        } catch(NumberFormatException nfe) {
        } catch(NullPointerException npe) {   // easiest way to code it
        }

        // if a set of chars is passed in, and no end has been 
        // passed in, then make the end be the end of the set
        if( (setChrs != null) && (ed == 0) ) {
            ed = setChrs.length - 1;
        }

        try {
            ct = NumberUtils.createNumber(count).intValue();
        } catch(NumberFormatException nfe) {
        } catch(NullPointerException npe) {   // easiest way to code it
        }
        return RandomStringUtils.random(ct, st, ed, letters, numbers, setChrs);
    }

    public void initAttributes() {
        this.count = null;
        this.start = null;
        this.end   = null;
        this.type  = UNICODE;
    }


}
