/*
 * $Header: /home/cvs/jakarta-taglibs/string/src/org/apache/taglibs/string/ReplaceTag.java,v 1.9 2003/04/27 04:34:13 bayard Exp $
 * $Revision: 1.9 $
 * $Date: 2003/04/27 04:34:13 $
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 */
package org.apache.taglibs.string;

import org.apache.commons.lang.StringUtils;
import org.apache.commons.lang.NumberUtils;
import javax.servlet.jsp.JspException;

/**
 * Replace a specified substring with another string.
 * A number of times may be specified to say how many times 
 * to replace the substring.
 *
 * <dl>
 * <dt>replace</dt><dd>
 *             Value to find to replace.
 *             Required.
 * </dd>
 * <dt>with</dt><dd>
 *             Value to replace with.
 *             Required.
 * </dd>
 * <dt>count</dt><dd>
 *             Number of times to replace.
 *             Default is all matches.
 * </dd>
 * <dt>newlineToken</dt><dd>
 *             Token to use as a newline.
 *             This gets around the pain of haivng a literal newline in the jsp.
 *             By default this is ignored.
 * </dd>
 * <dt>carriageToken</dt><dd>
 *             Token to use as a carriage return.
 *             This gets around the pain of haivng a literal carriage return 
 *             in the jsp. 
 *             By default this is ignored.
 * </dd>
 * </dl>
 * 
 * @author bayard@generationjava.com
 */
public class ReplaceTag extends StringTagSupport {

    private String count;
    private String replace;
    private String with;
    private String newlineToken;
    private String carriageToken;

    public ReplaceTag() {
        super();
    }

    /**
     * Get the replace property
     *
     * @return String property
     */
    public String getReplace() {
        return this.replace;
    }

    /**
     * Set the replace property
     *
     * @param replace String property
     */
    public void setReplace(String replace) {
        this.replace = replace;
    }

    /**
     * Get the newlineToken property
     *
     * @return String property
     */
    public String getNewlineToken() {
        return this.newlineToken;
    }

    /**
     * Set the newlineToken property
     *
     * @param newlineToken String property
     */
    public void setNewlineToken(String newlineToken) {
        this.newlineToken = newlineToken;
    }

    /**
     * Get the carriageToken property
     *
     * @return String property
     */
    public String getCarriageToken() {
        return this.carriageToken;
    }

    /**
     * Set the carriageToken property
     *
     * @param carriageToken String property
     */
    public void setCarriageToken(String carriageToken) {
        this.carriageToken = carriageToken;
    }


    /**
     * Get the with property
     *
     * @return String property
     */
    public String getWith() {
        return this.with;
    }

    /**
     * Set the with property
     *
     * @param with String property
     */
    public void setWith(String with) {
        this.with = with;
    }


    /**
     * Get the count property
     *
     * @return String property
     */
    public String getCount() {
        return this.count;
    }

    /**
     * Set the count property
     *
     * @param count String property
     */
    public void setCount(String count) {
        this.count = count;
    }



    public String changeString(String text) throws JspException {
        String repl = replace;
        String wit = with;
        if(this.carriageToken != null) {
            repl = StringUtils.replace(replace, this.carriageToken, "\r");
            wit = StringUtils.replace(with, this.carriageToken, "\r");
        }
        if(this.newlineToken != null) {
            repl = StringUtils.replace(replace, this.newlineToken, "\n");
            wit = StringUtils.replace(with, this.newlineToken, "\n");
        }
        int ct = -1;
        if(count != null) {
            ct = NumberUtils.stringToInt(count);
        }
        return StringUtils.replace(text, repl, wit, ct);
    }

    public void initAttributes() {

        this.replace = null;

        this.with = null;

        this.count = null;

        this.newlineToken = null;
        this.carriageToken = null;

    }

}
