/*
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:  
 *       "This product includes software developed by the 
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * [Additional notices, if required by prior licensing conditions]
 *
 */ 
package org.apache.taglibs.utility.basic;


import javax.servlet.*;
import javax.servlet.jsp.*;
import javax.servlet.jsp.tagext.*;

import java.io.*;
import java.util.StringTokenizer;

/**
 * Add Javascript function to validate form input.
 *
 * Author Mandar Raje.
 */

public class ValidateTag extends BodyTagSupport {
    
    // Name of the form.
    String name;

    // Required fields separeted by comma.
    String []reqdFields;

    // Name of the Javascript function to be generated.
    String method;
    
    public void setName(String val) {
        this.name = val;
    }

    // Parse the mandatory fields and store them into an array.
    public void setReqdFields(String val) {
	StringTokenizer list = new StringTokenizer(val,",");
	reqdFields = new String [list.countTokens()];
	int i = 0;
	while (list.hasMoreTokens()) {
	    reqdFields[i++] = (String) list.nextToken();
	}
    }

    public void setMethod(String name) {
	this.method = name;
    }
    
    public int doStartTag() {
	return EVAL_BODY_TAG;
    }

    /**
     * I am going to take an easy way out. Instead of parsing the HTML
     * form tag and inserting onSubmit="methodName", I will shift the
     * burden to the custom-tag user. This is not ideal, but hey, 
     * something is better than nothing.
     **/
     
    public int doAfterBody() throws JspException {

	try {
	    if (reqdFields.length == 0) return SKIP_BODY;
	    
	    // Javascript needs to be generated.
	    generateScript(bodyContent);
	    
	    bodyContent.writeOut(bodyContent.getEnclosingWriter());
	    return SKIP_BODY;
	    
	} catch (IOException iox) {
	    throw new JspException(iox.getMessage());
	}
    }

    /**
     * Again I am going to take an easy way out.
     * I am not generating the code per reqd. field.
     * If you fail to provide a value for a reqd. field
     * all it is going to say is "You have not filled
     * out all the mandatory fields" without giving out
     * the exact name for the field(s).
     * This can be improved (after I am back from vacation).
     **/
    
    public void generateScript(BodyContent out) throws IOException {

	out.println("<SCRIPT LANGUAGE=\"JavaScript\">");
	out.println("function " + this.method + "()");
	out.println("  {");
	out.println("    formObj = document." + this.name + ";");
	out.print("    if (");
	for(int i = 0; i < reqdFields.length; i++) {
	    if (i!= 0) out.println(" || ");
	    out.print("(formObj." + reqdFields[i] + ".value == \"\")");
	}
	out.println(")");
	out.println("     {");
	out.println("       alert(\"You have not filled out mandatory fields\");");
	out.println("       return false;");
	out.println("     }");
	out.print("    else return true; ");
	out.println("  }");
	out.println("</SCRIPT>");
    }
}

    
        
    
