/*
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:  
 *       "This product includes software developed by the 
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * [Additional notices, if required by prior licensing conditions]
 *
 */ 
package org.apache.taglibs.utility.lang;

import java.lang.reflect.Method;
import java.beans.*;
import java.io.IOException;

import javax.servlet.*;
import javax.servlet.http.HttpSession;
import javax.servlet.jsp.*;
import javax.servlet.jsp.tagext.*;

/**
 * Generate code for useBean compatible with JSP 0.92.  If the bean includes
 * a method with the signature:
 * <pre>
 *	public void processRequest(ServletRequest request);
 * </pre>
 * this method is called when the useBean tag is processed, with an argument
 * representing the servlet request currently being serviced.
 *
 * @author Mandar Raje
 */

public class UseBeanTag extends BodyTagSupport {
  
    String id;
    String scope = "page";
    String classname;
    String type;
    String beanName;
    boolean processRequest = false;
    boolean init = false;
    Object beanObject;

    public void setId(String val) {
	this.id = val;
    }
    
    public void setScope(String val) {
	this.scope = val;
    }

    public void setClassname(String val) {
	this.classname = val;
    }

    public void setType(String val) {
	this.type = val;
    }

    public void setBeanName(String val) {
	this.beanName = val;
    }

    public void setProcessRequest(boolean val) {
	this.processRequest = val;
    }

    public void setInit(boolean val) {
	this.init = val;
    }

    public void checkSyntax() throws JspException {
	
	    // Check for mandatory attributes:
	    if ( id == null ) {
		throw new JspException("useBean: id is missing");
	    }
	    
	    if (classname == null && type == null) {
		throw new JspException ("useBean: both class and name attribute" +
					      " can't be unspecified");
	    }

	    if (classname != null && beanName != null) {
		throw new JspException ("useBean: can't specify both class and" +
					      " beanName");
	    }	     
    }
  
    public int doStartTag() throws JspException { 

	    // check the syntax for useBean.
	    checkSyntax();
	    String clas = (this.classname == null)? type : this.classname;

	    try {
		
		// useBean idiosyncrasies.
		if (type == null) type = clas;
		
		if (scope == null || scope.equals ("page")) {
		    
		    // locate the object.
		    beanObject = pageContext.getAttribute(id);

		    if (beanObject != null) {
			if (!Class.forName(type).isInstance(beanObject))
			    throw new JspException("ClassCastException: " +
						   id + "  " + clas);
			return SKIP_BODY;
		    }
		    
		    synchronized(pageContext) {
			beanObject = createBean();
			if (processRequest ==  true)
			    processReq(beanObject);
			pageContext.setAttribute(id, beanObject);
			setInit(true);
		    }
		    
		} else  if (scope.equals ("request")) {
		    
		    // locate the object.
		    beanObject =
			pageContext.getAttribute(id, pageContext.REQUEST_SCOPE);

		    if (beanObject != null) {
			if (!Class.forName(type).isInstance(beanObject))
			    throw new JspException("ClassCastException: " +
						   id + "  " + clas);
			return SKIP_BODY;
		    }
		    
		    synchronized((ServletRequest)
				 pageContext.findAttribute(pageContext.REQUEST)) {
			beanObject = createBean();
			if (processRequest == true)
			    processReq(beanObject);
			pageContext.setAttribute(id, beanObject);
			setInit(true);
		    }		
		    
		} else if (scope.equals ("session")) {
		    
		    // locate the object.
		    beanObject = 
			pageContext.getAttribute(id, pageContext.SESSION_SCOPE);

		    if (beanObject != null) {
			if (!Class.forName(type).isInstance(beanObject))
			    throw new JspException("ClassCastException: " +
						   id + "  " + clas);
			return SKIP_BODY;
		    }
		    
		    synchronized((HttpSession)
				 pageContext.findAttribute(pageContext.SESSION)) {
			beanObject = createBean();
			if (processRequest == true)
			    processReq(beanObject);
			pageContext.setAttribute(id, beanObject);
			setInit(true);
		    }				
		    
		} else if (scope.equals ("application")) {
		    
		    // locate the object.
		    beanObject = 
			pageContext.getAttribute(id, pageContext.APPLICATION_SCOPE);

		    if (beanObject != null) {
			if (!Class.forName(type).isInstance(beanObject))
			    throw new JspException("ClassCastException: " +
						   id + "  " + clas);
			return SKIP_BODY;
		    }
		    
		    synchronized((ServletContext)
				 pageContext.findAttribute(pageContext.APPLICATION)) {
			beanObject = createBean();
			if (processRequest == true)
			    processReq(beanObject);
			pageContext.setAttribute(id, beanObject);
			setInit(true);
		    }		
		    
		}
		
		// Check if the body needs to be evaluated.
		if (init == false) return SKIP_BODY;
		else return EVAL_BODY_TAG;
	    } catch (Exception ex) {
		throw new JspException(ex.getMessage());
	    }
    }

    private Object createBean() throws JspException {

	// Create the bean only if classname is specified.
	// Otherwise bean must be located in the scope.
	try {
	    if (classname != null || beanName != null) {
		String className = (beanName != null) ? beanName : classname;
		return (Beans.instantiate
			(this.getClass().getClassLoader(),className));
	    }
	    else {
		throw new JspException ("useBean: bean must be located in" +
					" the given scope.");
	    }
	} catch (IOException iox) {
	    throw new JspException ("useBean: IO error while instantiating bean" +
				       iox.getMessage());
	} catch (ClassNotFoundException cnfex) {
	    throw new JspException ("useBean: class " + classname + " not found.");
	}
    }

    public int doAfterBody() {
	return SKIP_BODY;
    }

    public void processReq(Object bean) throws JspException {
	try {
	    Class clazz = Class.forName("javax.servlet.ServletRequest");
	    Method method = bean.getClass().getMethod("processRequest",
						  new Class[] { clazz });
	    method.invoke(bean, new Object[] {
		pageContext.findAttribute(pageContext.REQUEST) });
	    
	} catch(Exception ex) {
	    throw new JspException(ex.getMessage());
	}
    }
}

