/*
 * $Id: ExportTag.java,v 1.1 2000/07/03 19:30:00 craigmcc Exp $
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:  
 *       "This product includes software developed by the 
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * [Additional notices, if required by prior licensing conditions]
 *
 */ 

package org.apache.taglibs.xsl;


import java.io.BufferedInputStream;
import java.io.InputStreamReader;
import java.net.URL;
import java.net.URLConnection;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.jsp.JspException;
import javax.servlet.jsp.JspWriter;
import javax.servlet.jsp.PageContext;
import javax.servlet.jsp.tagext.TagSupport;


/**
 * Export the content of the specified JSP bean (in the specified scope)
 * to our output writer, presumably after modifications have been completed.
 *
 * @author Craig R. McClanahan
 * @version $Revision: 1.1 $ $Date: 2000/07/03 19:30:00 $
 */

public class ExportTag extends TagSupport {


    // ------------------------------------------------------------- Properties


    /**
     * The name of the scripting variable to be accessed.
     */
    private String name = null;

    public String getName() {
	return (this.name);
    }

    public void setName(String name) {
	this.name = name;
    }


    /**
     * The scope in which the scripting variable should be accessed.
     */
    private String scope = "page";

    public String getScope() {
	return (this.scope);
    }

    public void setScope(String scope) {
	this.scope = scope;
    }


    // --------------------------------------------------------- Public Methods


    /**
     * Export the contents of the specified JSP bean in the specified scope
     * to our output writer.
     *
     * @exception JspException if a JSP error occurs
     */
    public int doStartTag() throws JspException {

	// Validate the value of the "scope" attribute
	int scopeId = 0;
	if ("page".equalsIgnoreCase(scope))
	    scopeId = PageContext.PAGE_SCOPE;
	else if ("request".equalsIgnoreCase(scope))
	    scopeId = PageContext.REQUEST_SCOPE;
	else if ("session".equalsIgnoreCase(scope))
	    scopeId = PageContext.SESSION_SCOPE;
	else if ("application".equalsIgnoreCase(scope))
	    scopeId = PageContext.APPLICATION_SCOPE;
	else
	    throw new JspException("Invalid scope value '" + scope + "'");

	// Acquire the bean we will be rendering
	Object bean = pageContext.getAttribute(name, scopeId);
	if (bean == null)
	    throw new JspException("Missing JSP bean '" + name + "'");
	;	// FIXME - Special handling for DOM, InputSource, etc.?
	if (!(bean instanceof String))
	    bean = bean.toString();

	// Render the contents of this bean to our writer
	try {
	    JspWriter out = pageContext.getOut();
	    out.print((String) bean);
	} catch (Exception e) {
	    throw new JspException("Error writing output: " + e.toString());
	}

	// Skip the body of this tag (which should be empty anyway)
	return (SKIP_BODY);

    }


}
