/*
 * $Id: ImportTag.java,v 1.1 2000/07/03 19:30:03 craigmcc Exp $
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:  
 *       "This product includes software developed by the 
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * [Additional notices, if required by prior licensing conditions]
 *
 */ 

package org.apache.taglibs.xsl;


import java.io.BufferedInputStream;
import java.io.InputStreamReader;
import java.net.URL;
import java.net.URLConnection;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.jsp.JspException;
import javax.servlet.jsp.PageContext;
import javax.servlet.jsp.tagext.TagSupport;


/**
 * Import the content of the specified page, and assign it (as a String)
 * to the specified scripting variable in the specified scope.
 *
 * @author Craig R. McClanahan
 * @version $Revision: 1.1 $ $Date: 2000/07/03 19:30:03 $
 */

public class ImportTag extends TagSupport {


    // ------------------------------------------------------------- Properties


    /**
     * The name of the scripting variable to be created.
     */
    private String id = null;

    public String getId() {
	return (this.id);
    }

    public void setId(String id) {
	this.id = id;
    }


    /**
     * The URI of the page or servlet to be included.
     */
    private String page = null;

    public String getPage() {
	return (this.page);
    }

    public void setPage(String page) {
	this.page = page;
    }


    /**
     * The scope in which the scripting variable should be created.
     */
    private String scope = "page";

    public String getScope() {
	return (this.scope);
    }

    public void setScope(String scope) {
	this.scope = scope;
    }


    // --------------------------------------------------------- Public Methods


    /**
     * Import the specified page and assign it to the specified bean
     * in the specified scope.
     *
     * @exception JspException if a JSP error occurs
     */
    public int doStartTag() throws JspException {

	// Validate the format of the "page" attribute
	// FIXME - deal with relative URIs like <jsp:include> does
	if (!page.startsWith("/"))
	    throw new JspException("Page value must start with '/'");

	// Validate the value of the "scope" attribute
	int scopeId = 0;
	if ("page".equalsIgnoreCase(scope))
	    scopeId = PageContext.PAGE_SCOPE;
	else if ("request".equalsIgnoreCase(scope))
	    scopeId = PageContext.REQUEST_SCOPE;
	else if ("session".equalsIgnoreCase(scope))
	    scopeId = PageContext.SESSION_SCOPE;
	else if ("application".equalsIgnoreCase(scope))
	    scopeId = PageContext.APPLICATION_SCOPE;
	else
	    throw new JspException("Invalid scope value '" + scope + "'");

	// Set up the buffer to which we will write (if id is set)
	StringBuffer sb = new StringBuffer();

	// Set up a URLConnection to read the requested page.  We cannot use
	// PageContext.include() because it writes directly to the output
	// stream that goes back to the client. :-(
	HttpServletRequest request =
	    (HttpServletRequest) pageContext.getRequest();
	StringBuffer url = new StringBuffer();
	url.append(request.getScheme());
	url.append("://");
	url.append(request.getServerName());
	if (request.getServerPort() != 80) {
	    url.append(':');
	    url.append(request.getServerPort());
	}
	if (request.getContextPath() != null)
	    url.append(request.getContextPath());
	url.append(page);	// FIXME - deal with relative page references
	URLConnection conn = null;
	try {
	    conn = (new URL(url.toString())).openConnection();
	    conn.setAllowUserInteraction(false);
	    conn.setDoInput(true);
	    conn.setDoOutput(false);
	    conn.connect();
	} catch (Exception e) {
	    throw new JspException("Error opening connection: " +
				   e.toString());
	}

	// Copy the contents of this URL
	try {
	    BufferedInputStream is =
		new BufferedInputStream(conn.getInputStream());
	    InputStreamReader in = new InputStreamReader(is);
	    while (true) {
		int ch = in.read();
		if (ch < 0)
		    break;
		sb.append((char) ch);
	    }
	} catch (Exception e) {
	    throw new JspException("Error reading connection: " +
				   e.toString());
	}

	// Store the accumulated buffer as a bean
	pageContext.setAttribute(id, sb.toString(), scopeId);

	// Skip the body of this tag (which should be empty anyway)
	return (SKIP_BODY);

    }


}
