package com.taglib.authentication;

import com.taglib.util.DbConnectionFactory;

import java.security.MessageDigest;
import java.sql.PreparedStatement;
import java.sql.Connection;
import java.sql.ResultSet;

import java.security.NoSuchAlgorithmException;
import java.sql.SQLException;

public class DbUserBase {
  static private DbConnectionFactory connectionFactory = DbConnectionFactory.getInstance();

  static public boolean validate (String username, String password) {
    password = encodePassword(password);
    Connection conn = null;
    PreparedStatement stmt = null;
    ResultSet rset = null;
    try {
      conn = connectionFactory.getConnection();
      stmt = conn.prepareStatement(SQL_VALIDATE);
      stmt.setString(1, username);
      stmt.setString(2, password);
      rset = stmt.executeQuery();
      if (rset.next()) {
        long id = rset.getLong(1);
        return true;
      }
    }
    catch (SQLException e) {
      System.out.println("ERR: Unable to validate " + username + " against db (" + e + ")");
    }
    finally {
      if (rset != null) try { rset.close(); } catch (SQLException e) {}
      if (stmt != null) try { stmt.close(); } catch (SQLException e) {}
      if (conn != null) try { conn.close(); } catch (SQLException e) {}
    }
    return false;
  }

  static public String encodePassword (String pwd) {
    try {
      MessageDigest md5 = MessageDigest.getInstance("MD5");
      md5.update(pwd.getBytes());
      byte[] hash = md5.digest();
      return toHexString(hash);
    }
    catch (NoSuchAlgorithmException e) {
      System.out.println("ERR: unable to encode password (" + e.toString() + ")");
      return "";
    }
  }
  final static private String toHexString (byte[] n) {
    StringBuffer hex = new StringBuffer(2*n.length);
    for (int i = 0; i < n.length; ++i) {
      byte b = n[i];
      if ((b >= 0) && (b <= 15)) {
	hex.append("0");
      }
      // note: & 0xff is used to reset high bits
      hex.append(Integer.toHexString(b & 0xff));
    }
    return hex.toString();
  }

  static public UserBean fetch (String username) {
    UserBean user = new UserBean();
    populate(user, username);
    return user;
  }
  static public UserBean fetch (long id) {
    UserBean user = new UserBean();
    populate(user, id);
    return user;
  }

  static public void populate (UserBean bean, String username) {
    populate(bean, username, fetchId(username));
  }
  static private void populate (UserBean bean, long id) {
    populate(bean, fetchUsername(id), id);
  }
  static private void populate (UserBean bean, String username, long id) {
    Connection conn = null;
    PreparedStatement stmt = null;
    ResultSet rset = null;
    try {
      conn = connectionFactory.getConnection();
      stmt = conn.prepareStatement(SQL_FETCH_USER_DATA);
      stmt.setLong(1, id);
      rset = stmt.executeQuery();
      if (rset.next()) {
        String name = rset.getString(1);
        String email = rset.getString(2);
        bean.setUsername(username);
        bean.setId(id);
        bean.setFullName(name);
        bean.setEmailAddress(email);
        assignRoles(bean);
      }
    }
    catch (SQLException e) {
      System.out.println("ERR: Unable to populate user " + id + " from db (" + e + ")");
    }
    finally {
      if (rset != null) try { rset.close(); } catch (SQLException e) {}
      if (stmt != null) try { stmt.close(); } catch (SQLException e) {}
      if (conn != null) try { conn.close(); } catch (SQLException e) {}
    }
  }

 static private long fetchId (String username) {
    Connection conn = null;
    PreparedStatement stmt = null;
    ResultSet rset = null;
    try {
      conn = connectionFactory.getConnection();
      stmt = conn.prepareStatement(SQL_FETCH_USER_ID);
      stmt.setString(1, username);
      rset = stmt.executeQuery();
      if (rset.next()) {
        long id = rset.getLong(1);
        return id;
      }
    }
    catch (SQLException e) {
      System.out.println("ERR: Unable to populate " + username + " from db (" + e + ")");
    }
    finally {
      if (rset != null) try { rset.close(); } catch (SQLException e) {}
      if (stmt != null) try { stmt.close(); } catch (SQLException e) {}
      if (conn != null) try { conn.close(); } catch (SQLException e) {}
    }
   return UserBean.INVALID_ID;
  }
  static private String fetchUsername (long id) {
    Connection conn = null;
    PreparedStatement stmt = null;
    ResultSet rset = null;
    try {
      conn = connectionFactory.getConnection();
      stmt = conn.prepareStatement(SQL_FETCH_USERNAME);
      stmt.setLong(1, id);
      rset = stmt.executeQuery();
      if (rset.next()) {
        String username = rset.getString(1);
        return username;
      }
    }
    catch (SQLException e) {
      System.out.println("ERR: Unable to retrieve username for " + id + " from db (" + e + ")");
    }
    finally {
      if (rset != null) try { rset.close(); } catch (SQLException e) {}
      if (stmt != null) try { stmt.close(); } catch (SQLException e) {}
      if (conn != null) try { conn.close(); } catch (SQLException e) {}
    }
    return null;
  }

  static private void assignRoles (UserBean user) throws SQLException {
    Connection conn = null;
    PreparedStatement pstmt = null;
    ResultSet rset = null;
    try {
      conn = connectionFactory.getConnection();
      pstmt = conn.prepareStatement(SQL_FETCH_USER_ROLES);
      pstmt.setLong(1, user.getId());
      rset = pstmt.executeQuery();
      while (rset.next()) {
        user.addRole(rset.getString(1));
      }
    }
    finally {
      try { if (rset != null) rset.close(); } catch (SQLException e) {}
      try { if (pstmt != null) pstmt.close(); } catch (SQLException e) {}
      try { if (conn != null) conn.close(); } catch (SQLException e) {}
    }
  }

  final static private String SQL_VALIDATE = "SELECT id FROM users WHERE login = ? AND password = ?";
  final static private String SQL_FETCH_USER_ID = "SELECT id FROM users WHERE login = ?";
  final static private String SQL_FETCH_USERNAME = "SELECT login FROM users WHERE id = ?";
  final static private String SQL_FETCH_USER_DATA = "SELECT name, email_address FROM profiles WHERE id = ?";
  final static private String SQL_FETCH_USER_ROLES =
    "SELECT define_roles.role_name FROM define_roles INNER JOIN user_roles ON define_roles.role_id = user_roles.role_id WHERE user_roles.user_id = ?";

  static public void main (String args[]) {
    int stop = args.length;
    for (int i = 0; i < stop; ++i) {
      String unencoded = args[i];
      System.out.println(unencoded + " -> " + encodePassword(unencoded));
    }
  }
}