package com.taglib.authentication;

import javax.servlet.ServletConfig;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import javax.servlet.ServletException;
import java.io.IOException;

public class LoginServlet extends HttpServlet {
  final private static String USERNAME_REQPARM = "username";
  final private static String PASSWORD_REQPARM = "password";

  private String welcomePage;
  private String loginPage;
  private UserSessionManager sessionMgr = UserSessionManager.getInstance();
  private UserDataManager userMgr;

  public void init (ServletConfig config) throws ServletException {
    welcomePage = config.getInitParameter("welcomePage");
    if (welcomePage == null)
      throw new ServletException("The welcomePage init parameter"
				 + " must be specified.");
    loginPage = config.getInitParameter("loginPage");
    if (loginPage == null)
      throw new ServletException("The loginPage init parameter"
				 + " must be specified.");
    String userDataMgrClassname =
      config.getInitParameter("userDataManager");
    if (userDataMgrClassname == null)
      throw new ServletException("The userDataManager init parameter" + " must be specified.");
    userMgr = (UserDataManager) makeInstance(userDataMgrClassname, UserDataManager.class, "user data manager");
  }
  private Object makeInstance (String classname,
			       Class interfaceClass, String description)
    throws ServletException {
    try {
      Class klass = Class.forName(classname);
      if (! interfaceClass.isAssignableFrom(klass)) {
	throw new ServletException(classname
				   + " does not implement required"
				   + " interface"
				   + interfaceClass.getName()
				   + " for " + description + ".");
      }
      return klass.newInstance();
    }
    catch (ClassNotFoundException e) {
      throw new ServletException("Unable to instantiate" + description + ".", e);
    }
    catch (InstantiationException e) {
      throw new ServletException("Unable to instantiate" + description + ".", e);
    }
    catch (IllegalAccessException e) {
      throw new ServletException("Unable to instantiate" + description + ".", e);
    }
  }

  public void doPost (HttpServletRequest request, HttpServletResponse response)
    throws ServletException, IOException {
    String username = request.getParameter(USERNAME_REQPARM);
    if (username == null)
      throw new ServletException("No username specified.");
    String password = request.getParameter(PASSWORD_REQPARM);
    if (password == null)
      throw new ServletException("No password specified.");
    if (userMgr.validate(username, password)) {
      doLoginSuccess(request, response, username);
    } else {
      doLoginFailure(request, response, username);
    }
  }
  private void doLoginFailure (HttpServletRequest request, HttpServletResponse response, String username)
    throws IOException {
    String loginURL = loginPage + "?retry=true&username=" + username;
    UserSessionManager.doRedirect(request, response, loginURL);
  }
  private void doLoginSuccess (HttpServletRequest request, HttpServletResponse response, String username)
    throws IOException {
    UserBean user = userMgr.lookupUserData(username);
    HttpSession session = request.getSession(true);
    sessionMgr.setSessionUser(session, user);
    String targetPage = sessionMgr.getSessionLoginTarget(session, true);
    if (targetPage == null) {
      UserSessionManager.doRedirect(request, response, welcomePage);
    } else {
      targetPage = response.encodeRedirectURL(targetPage);
      response.sendRedirect(targetPage);
    }
  }
}
