package com.taglib.authentication;

import javax.servlet.http.HttpSession;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import java.io.IOException;

public class UserSessionManager {
  final static public String SESSION_USER_ATTRIBUTE = "user";
  final static public String SESSION_LOGINTARGET_ATTRIBUTE = "loginTarget";

  static private UserSessionManager INSTANCE = new UserSessionManager();
  private UserSessionManager () {}

  static public UserSessionManager getInstance () {
    return INSTANCE;
  }

  public void setSessionUser (HttpSession session, UserBean user) {
    session.setAttribute(SESSION_USER_ATTRIBUTE, user);
    if (user.hasRole("admin")) {
      session.setMaxInactiveInterval(60 * 60 * 8);
    } else {
      session.setMaxInactiveInterval(60 * 15);
    }
  }
  public void removeSessionUser (HttpSession session) {
    session.removeAttribute(SESSION_USER_ATTRIBUTE);
  }
  public UserBean getSessionUser (HttpSession session) {
    return (UserBean) session.getAttribute(SESSION_USER_ATTRIBUTE);
  }

  public void removeSessionUser (HttpServletRequest request) {
    removeSessionUser(request.getSession(true));
  }
  public UserBean getSessionUser (HttpServletRequest request) {
    return getSessionUser(request.getSession(true));
  }

  public void setSessionLoginTarget (HttpSession session, String loginTarget) {
    session.setAttribute(SESSION_LOGINTARGET_ATTRIBUTE, loginTarget);
  }
  public String getSessionLoginTarget (HttpSession session, boolean clear) {
    String target =
      (String) session.getAttribute(SESSION_LOGINTARGET_ATTRIBUTE);
    if (clear) session.removeAttribute(SESSION_LOGINTARGET_ATTRIBUTE);
    return target;
  }

  static public void doRedirect (HttpServletRequest request,
				 HttpServletResponse response, String url)
    throws java.io.IOException {
    String redirect =
      response.encodeRedirectURL(request.getContextPath() + url);
    response.sendRedirect(redirect);
  }
}
