<%@ taglib prefix="c" uri="http://java.sun.com/jstl/core" %>
<%@ taglib prefix="tldc" uri="/tldc" %>

<font class="blogTitle">
 <tldc:escapeXml><c:import></tldc:escapeXml> vs. <tldc:escapeXml><jsp:include></tldc:escapeXml>
</font>

<p>
With the introduction of JSTL, JSP authors are faced with a quandary:
When incorporating content into a page from some other resource, should one use
<tldc:escapeXml style="code"><c:import></tldc:escapeXml> or
<tldc:escapeXml style="code"><jsp:include></tldc:escapeXml>?
</p>

<p>
Well, it depends.
</p>

<p>
As it stands now, the primary advantage of
<tldc:escapeXml style="code"><jsp:include></tldc:escapeXml> is its control
over buffer flushing.
JSTL's <tldc:escapeXml style="code"><c:import></tldc:escapeXml> has no such
control, and calling it will automatically flush the output buffer. So there's
no equivalent to the following with <tldc:escapeXml style="code"><c:import></tldc:escapeXml>:
<blockquote>
  <tldc:escapeXml style="code"><jsp:include page="/content.jsp" flush="false"/></tldc:escapeXml>
</blockquote>
So if your application needs to be careful about flushing the buffer (typically
because of the negative impact of flushing the buffer on error handling),
then <tldc:escapeXml style="code"><jsp:include></tldc:escapeXml> is to be
preferred.
</p>

<p>
If buffer flushing is not a major concern, however, then it's hard to
dispute that <tldc:escapeXml style="code"><c:import></tldc:escapeXml>
is much more powerful than <tldc:escapeXml style="code"><jsp:include></tldc:escapeXml>.
The ability of <tldc:escapeXml style="code"><c:import></tldc:escapeXml> to
fetch content from other application contexts, other servers, and using
multiple protocols provides a great deal of flexibility to JSP developers.
And let's not forget that, at least until JSP 2.0 is finalized and becomes
generally available, <tldc:escapeXml style="code"><jsp:include></tldc:escapeXml>
cannot take advantage of the JSTL expression language. Using an expression
to specify the resource to be included is, at the moment, solely the purview
of <tldc:escapeXml style="code"><c:import></tldc:escapeXml>:
<blockquote>
<tldc:escapeXml style="code"><c:choose>
	<c:when test="${favoriteColor eq 'blue'}">
		<c:set var="inc" value="/blueContent.jsp"/>
	</c:when>
	<c:otherwise>
		<c:set var="inc" value="/redContent.jsp"/>
	</c:otherwise>
</c:choose>
<c:import url="{$inc}"/></tldc:escapeXml>
</blockquote>
If you're already managing page data <i>via</i> the expression language,
then there are clear advantages to
<tldc:escapeXml style="code"><c:import></tldc:escapeXml> over
<tldc:escapeXml style="code"><jsp:include></tldc:escapeXml>
</p>

<p>
Blah blah blah blah blah.
Blah blah blah blah blah blah blah blah.
Blah blah.
Blah blah blah blah blah blah blah blah.
Blah blah blah blah blah blah blah.
Blah blah blah blah blah.
Blah blah blah blah blah blah.
Blah blah blah blah blah.
</p>
