/*
 * Copyright 2003 Sun Microsystems, Inc. All rights reserved.
 * SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */
 
import java.io.FileInputStream;
import java.io.IOException;
import java.math.BigDecimal;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import javax.xml.bind.Unmarshaller;
import java.util.List;

// import java content classes generated by binding compiler
import example.*;

/*
 * $Id: Main.java,v 1.1 2003/09/24 19:38:21 jendrock Exp $
 *
 * Copyright 2003 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */
 
public class Main {
    
    // This sample application demonstrates how to modify a collection
    // in a java content tree and marshal it back to xml data
    
    public static void main( String[] args ) {
        try {
            // create a JAXBContext capable of handling classes generated into
            // the example package
            JAXBContext jc = JAXBContext.newInstance( "example" );
            
            // create an Unmarshaller
            Unmarshaller u = jc.createUnmarshaller();
            
            // unmarshal a FooBar instance document into a tree of Java content
            // objects composed of classes from the example package.
            FooBar fb = 
                (FooBar)u.unmarshal( new FileInputStream( "example.xml" ) );

	    // read and write unnamed model group bound to Java content interface
	    FooBar.PhoneNumberOrSpeedDial numOrSD = 
					fb.getPhoneNumberOrSpeedDial();
	    System.out.println("PhoneNumberOrSpeedDial value:" + numOrSD.getContent()+
		               " isPhoneNumber:" + numOrSD.isSetPhoneNumber() + 
		               " isSpeedDial:" + numOrSD.isSetSpeedDial());
            numOrSD.setSpeedDial(22);
	    
	    // read and write model group def.bound to Java content interface
            ModelGroupChoice mgc = fb.getModelGroupChoice();
	    System.out.println("ModelGroupChoice value:" + mgc.getContent()+
		               " isBool:" + mgc.isSetBool() + 
		               " isComment:" + mgc.isSetComment() + 
		               " isValue:" + mgc.isSetValue());
	    mgc.setBool(false);
	     
            // create a Marshaller and marshal to a file
            Marshaller m = jc.createMarshaller();
            m.setProperty( Marshaller.JAXB_FORMATTED_OUTPUT, Boolean.TRUE );
            m.marshal( fb, System.out );
            
        } catch( JAXBException je ) {
            je.printStackTrace();
        } catch( IOException ioe ) {
            ioe.printStackTrace();
        }
    }
}
