package sequoia;

import org.apache.log4j.Logger;

/**
 * Factory that creates and destroys <pre>Peon</pre>s.
 */
public class PeonFactory {
  static Logger logger = Logger.getLogger(Peon.class);

  private ConnectionFactory connectionFactory;
  protected String connectionFactoryImplementation;

  public PeonFactory(String connectionFactoryImplementation) {
    this.connectionFactoryImplementation = connectionFactoryImplementation;
    loadConnectionFactory();
  }

  protected void loadConnectionFactory()
    throws ConnectionFactoryInstantiationError
  {
    try {
      this.connectionFactory = (ConnectionFactory) Class.forName(connectionFactoryImplementation).newInstance();
    } catch (ClassNotFoundException e) {
      throw new ConnectionFactoryInstantiationError(e);
    } catch (InstantiationException e) {
      throw new ConnectionFactoryInstantiationError(e);
    } catch (IllegalAccessException e) {
      throw new ConnectionFactoryInstantiationError(e);
    }
  }


  protected void setConnectionFactoryImplementation(String implementation) {
    this.connectionFactoryImplementation = implementation;
  }
  protected String getConnectionFactoryImplementation() {
    return this.connectionFactoryImplementation;
  }

  public ConnectionFactory getConnectionFactory(){
    return connectionFactory;
  }

  public void setConnectionFactory(ConnectionFactory connectionFactory){
    this.connectionFactory = connectionFactory;
  }

  public Peon newPeon() {
    try {
      Peon peon = new Peon();
      peon.connection = connectionFactory.connect();
      return peon;
    } catch (Exception e) {
      logger.fatal("Unable to instantiate Peon:" + e.getMessage());
      throw new PeonInstantiationException("Failed to instantiate Peon", e);
    }
  }
}