package sequoia;

import java.sql.Connection;
import java.sql.SQLException;
import java.sql.ResultSet;
import java.sql.Statement;

public class Peon {
  protected Connection connection = null;

  protected ResultSet lastResultSet = null;
  protected Statement lastStatement = null;
  protected int lastUpdateCount = 0;

  public boolean executeQuery(String query) throws SQLException {
    Statement stm = null;
    lastResultSet = null;
    lastUpdateCount = 0;
    try {
      stm = connection.createStatement(
        ResultSet.TYPE_SCROLL_INSENSITIVE,
        ResultSet.CONCUR_READ_ONLY
      );
    } catch (AbstractMethodError e) {
      stm = connection.createStatement();
    }
    boolean resultType = stm.execute(query);
    if(resultType) {
      lastResultSet = stm.getResultSet();
    } else {
      lastUpdateCount = stm.getUpdateCount();
    }
    lastStatement = stm;
    return resultType;
  }

  protected void finalize() {
    try {
      synchronized(this) {
        if(lastResultSet != null) {
          lastResultSet.close();
          lastResultSet = null;
        }
        if(lastStatement != null) {
          lastStatement.close();
          lastStatement = null;
        }
        connection.close();
        connection = null;
      }
    } catch (SQLException e) {
      e.printStackTrace(System.err);
    } finally {
      if(lastResultSet != null) {
        try { lastResultSet.close(); } catch (SQLException e) { }
        lastResultSet = null;
      }
      if(lastStatement != null) {
        try { lastStatement.close(); } catch (SQLException e) { }
        lastStatement = null;
      }
      if(connection != null) {
        try {connection.close(); } catch (SQLException e) { }
        connection = null;
      }
    }
  }

  public void unsummon() {
    finalize();
  }

  public ResultSet getLastResultSet() {
      return lastResultSet;
  }

  public int getLastUpdateCount() {
    return lastUpdateCount;
  }
}